#!/bin/bash
ASCEND_SECLOG="/var/log/ascend_seclog"
logFile="${ASCEND_SECLOG}/ascend_install.log"
INSTALL_CACHE_DIR="/var/mini/install_cache"
FWM_DIR="/fw"
crldir="/root/ascend_check"
crltool="unknow"
crltool_bin="ascend_310_check.bin"
crlFile="ascend_310.crl"
curcrlFile="cur_ascend_310.crl"
pcie_tee="tee.bin"

CRL_OPERATION_LOGDIR="/var/log/upgradetool"
CRL_OPERATION_LOGPATH="${CRL_OPERATION_LOGDIR}/upgrade_crl.log"
LOG_LEVEL_SUGGESTION="SUGGESTION"
LOG_LEVEL_MINOR="MINOR"
LOG_LEVEL_MAJOR="MAJOR"
MODULE="DEVICE_CRL_CHECK"

start_time=$(date +"%Y-%m-%d %H:%M:%S")
log() {
    cur_date=`date +"%Y-%m-%d %H:%M:%S"`
    echo "[driver] [$cur_date] "$1 >> $logFile
}
logOperationCrl() {
    local operation="$1"
    local level="$2"
    local start_time="$3"
    local module="$4"
    local result="$5"

    if [ ! -d "${CRL_OPERATION_LOGDIR}" ]; then
        mkdir -p ${CRL_OPERATION_LOGDIR}
        chmod 700 ${CRL_OPERATION_LOGDIR}
    fi

    if [ ! -f "${CRL_OPERATION_LOGPATH}" ]; then
        touch ${CRL_OPERATION_LOGPATH}
        chmod 600 ${CRL_OPERATION_LOGPATH}
    fi

    echo "${operation} ${level} root ${start_time} 127.0.0.1 ${module} ${result}" >> ${CRL_OPERATION_LOGPATH}
}

#get current environment images crl when crl not exist in /root/ascend_check
gen_cur_image_crl_check_valid() {
    local ret=0
    log "[INFO]case one start: generate new images crl, check vaild and cms "

    if [ ! -f $crldir/$crlFile ] ;then
        touch $crldir/$crlFile
        chmod 400 $crldir/$crlFile
        log "[INFO]touch $crlFile success"
    fi
    if [ -f $crldir/$crltool_bin ]; then
         crltool=$crldir/$crltool_bin
    else
         crltool=$sourcedir/$crltool_bin
    fi
    $crltool 0 $sourcedir/tee.bin $crldir/$crlFile > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        log "[INFO]1.generate new images crl fail, direct install "
        rm -rf $crldir/$crlFile
        return 0
    fi
    log "[INFO]1.generate new images crl success , start check valid"
    #校验有效性，输入：2，文件名字 输出：0（正确），其他为非法
    $crltool 2  $crldir/$crlFile > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        log "[ERROR]2.check new images crl failed, stop install"
        rm -rf $crldir/$crlFile
        return 1
    fi
    log "[INFO]2.check new images crl success,start check cms"
    #验签，输入：3，较新的crl，要升级的images
    $crltool 3  $crldir/$crlFile  $sourcedir/tee.bin > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        log "[ERROR]3.check new images crl cms failed, stop install"
        rm -rf $crldir/$crlFile
        return 1
    fi
    log "[INFO]3.check new images crl cms success, update tool and crl"
    if [ -f $crldir/$crltool_bin ]; then
        rm -f $crldir/$crltool_bin
    fi
    cp -f $sourcedir/$crltool_bin $crldir/$crltool_bin
    chmod 500 $crldir/$crltool_bin
    cp -f $sourcedir/ascend_upgrade_crl.sh $crldir/ascend_upgrade_crl.sh
    chmod 500 $crldir/ascend_upgrade_crl.sh
    log "[INFO]4.update /root/ascend_check/$crltool_bin, ascend_upgrade_crl.sh and crl success"
    log "[INFO]case one end: generate new images crl, check vaild and cms "
    return 0
}
#获取新的imeages的crl文件，并和/root/ascend_check/下的crl文件比较（该路径下crl必定存在）
gen_cur_image_crl_compare_newer() {
    local ret=0
    local new_tool_flag=0
    log "[INFO]case two start: generate new images crl, compare newer and check cms "

    if [ ! -f $crldir/$crltool_bin ]; then
        cp -f $sourcedir/$crltool_bin $crldir/$crltool_bin
        chmod 500 $crldir/$crltool_bin
        cp -f $sourcedir/ascend_upgrade_crl.sh $crldir/ascend_upgrade_crl.sh
        chmod 500 $crldir/ascend_upgrade_crl.sh
        new_tool_flag=1
        log "[INFO]use new $crltool_bin and ascend_upgrade_crl.sh in runpackage"
        logOperationCrl "UPGRADE" "${LOG_LEVEL_MAJOR}" "${start_time}" "${MODULE}" "upgrade $crltool_bin and ascend_upgrade_crl.sh success"
    fi
    #解压run包images的crl文件，$curcrlFile，输入：类型(0:获取文件，1:比较,2检查有效性，3验签)，images,路径+文件名，输出：返回值0成功，其他表示失败
    touch $crldir/$curcrlFile
    chmod 400 $crldir/$curcrlFile
    $crldir/$crltool_bin 0 $sourcedir/tee.bin $crldir/$curcrlFile > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        log "[ERROR]1.generate new images crl fail, stop install"
        rm -rf $crldir/$curcrlFile
        return 1
    fi
    log "[INFO]1.generate new images crl success, need check if newer"
    #比较cur_$crlfile和$crlFile新旧，输入：类型(0:获取文件，1:比较，2检查有效性，3验签)，路径+new_images，路径+old_images   #输出：返回值，0(SCPS_SAME),1(SCPS_NEW),2(SCPS_OLD),3(SCPS_MIX)
    #如果cur较新，需要做验签，并更新镜像,否则停止安装。验签参数：3，较新的crl，要升级的images
    $crldir/$crltool_bin 1 $crldir/$curcrlFile $crldir/$crlFile > /dev/null 2>&1
    ret=$?
    if [ $ret -eq 0 ] || [ $ret -eq 1 ]; then
         log "[INFO]2.ret=$ret,$curcrlFile same or newer, start check cms"
         $crldir/$crltool_bin 3 $crldir/$curcrlFile $sourcedir/tee.bin > /dev/null 2>&1
         if [ $? -ne 0 ]; then
             log "[ERROR]3.check $curcrlFile cms failed, stop install"
             rm -rf $crldir/$curcrlFile
             return 1
         fi
         log "[INFO]3.check $curcrlFile cms  success, update tool and crl"
         rm -rf $crldir/$crlFile
         mv $crldir/$curcrlFile $crldir/$crlFile
         if [ $new_tool_flag -eq 0 ]; then
            rm -f $crldir/$crltool_bin
            cp -f $sourcedir/$crltool_bin $crldir/$crltool_bin
            chmod 500 $crldir/$crltool_bin
            cp -f $sourcedir/ascend_upgrade_crl.sh $crldir/ascend_upgrade_crl.sh
            chmod 500 $crldir/ascend_upgrade_crl.sh
            log "[INFO]update $crltool_bin and ascend_upgrade_crl.sh in runpackage"
            logOperationCrl "UPGRADE" "${LOG_LEVEL_MAJOR}" "${start_time}" "${MODULE}" "upgrade $crltool_bin, crl and ascend_upgrade_crl.sh success"
         fi
         log "[INFO]4.update $crltool_bin and crl success"
    elif [ $ret -eq 2 ] ; then
         log "[INFO]2.ret=$ret,$curcrlFile older, start check cms"
         rm -rf $crldir/$curcrlFile
         $crldir/$crltool_bin 3 $crldir/$crlFile $sourcedir/tee.bin > /dev/null 2>&1
         if [ $? -ne 0 ]; then
             log "[ERROR]3.check $crlFile cms failed,stop install"
             return 1
         fi
         log "[INFO]3.check $crlFile cms  success, not upgrade tool and crl"
    elif [ $ret -eq 5 ] || [ $ret -eq 4 ]; then
        log "[WARNING].ret=$ret,completeness valid check fail, stop drvier install ,need remove crl then upgrade"
        rm -rf $crldir/$curcrlFile
        return 1
    else
        log "[ERROR]2.ret=$ret,$curcrlFile not newer or both crl completeness valid check fail, stop update images"
        rm -rf $crldir/$curcrlFile
        return 1
    fi
    log "[INFO]case two end: generate new images crl, compare newer and check cms "
    return 0
}
#环境/root/ascend_check里不存在crl，去获取环境上old images的crl,并根据是否存在crl同新的images的crl比较
gen_old_image_crl() {
    log "[INFO]case three start: generate old images crl, then choose to case one or case two"
    if [ ! -f $FWM_DIR/$pcie_tee ];then
        #如果old images不存在走首次安装流程，校验新images并判断合法性
        gen_cur_image_crl_check_valid
        if [ $? -ne 0 ]; then
            log "[ERROR]gen_cur_image_crl_check_valid failed"
            return 1
        fi
        log "[INFO]gen_cur_image_crl_check_valid success"
        return 0
    fi
    touch $crldir/$crlFile
    chmod 400 $crldir/$crlFile
    log "[INFO]touch $crlFile success"
    if [ -f $crldir/$crltool_bin ]; then
         crltool=$crldir/$crltool_bin
    else
         crltool=$sourcedir/$crltool_bin
    fi
    $crltool 0 $FWM_DIR/$pcie_tee $crldir/$crlFile > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        log "[INFO]old images not exist crl, check new images, choose to case one "
        rm -rf $crldir/$crlFile
        gen_cur_image_crl_check_valid
        if [ $? -ne 0 ]; then
            log "[ERROR]gen_cur_image_crl_check_valid failed"
            return 1
        fi
        log "[INFO]gen_cur_image_crl_check_valid success"
        return 0
    else
        log "[INFO]old images exist crl, compare if cur crl newer,choose to case two"
        gen_cur_image_crl_compare_newer
        if [ $? -ne 0 ]; then
            log "[ERROR]gen_cur_image_crl_compare_newer failed"
            return 1
        fi
        log "[INFO]gen_cur_image_crl_compare_newer success"
        return 0
    fi
    log "[INFO]case three end: generate old images crl, then choose to case one or case two"
}


device_images_crl_check() {
    export LD_LIBRARY_PATH=$INSTALL_CACHE_DIR/driver:${LD_LIBRARY_PATH}
    crltool="$crldir/$crltool_bin"
    local ret=0
    if [ ! -d $crldir ]; then
        mkdir -p $crldir
        chmod 500 $crldir
        log "[INFO]mkdir $crldir success"
    fi
    if [ -f $crldir/$crlFile ]; then
         gen_cur_image_crl_compare_newer
         if [ $? -ne 0 ]; then
            log "[ERROR]gen_cur_image_crl_compare_newer failed"
            return 1
         fi
         log "[INFO]gen_cur_image_crl_compare_newer success"
         return 0
    else
        gen_old_image_crl
        if [ $? -ne 0 ]; then
            log "[ERROR]gen_old_image_crl fail, install failed"
            return 1
        fi
    fi
    return 0
}

sourcedir=$INSTALL_CACHE_DIR/driver
device_images_crl_check
if [ $? -ne 0 ]; then
    logOperationCrl "UPGRADE" "${LOG_LEVEL_MINOR}" "${start_time}" "${MODULE}" "upgrade $crltool_bin,crl and ascend_upgrade_crl.sh failed"
    exit 1
fi

exit 0
