#!/bin/bash
# run包安装解析公共脚本
# 解析filelist.csv文件，完成目录创建，文件复制，权限设置，文件删除等操作。

# db.info文件格式如下
# ---
# CommonLib|atc,fwkacllib
# Compiler|atc,fwkacllib
# ---
# 使用"|"分隔模块名和包名列表，包名列表使用","分隔
# 文件中块名保持以升序排序
PKG_DB_INFO_RELPATH="var/ascend_package_db.info"
# 缓存mod文件名
STASH_MOD_PATH="stash_mod.txt"
# stash_mod文件权限
STASH_FILE_MOD="600"
RESET_MOD="750"
# 总setenv文件权限
SETENV_MOD="555"
# 总setenv文件可写状态权限
SETENV_WRITEABLE_MOD="600"
# 总uninstall.sh文件权限
TOTAL_UNINSTALL_MOD="500"
# db.info文件权限
DB_INFO_MOD="640"

# 写日志
log() {
    local cur_date_="$(date +"%Y-%m-%d %H:%M:%S")"
    local log_type_="${1}"
    local msg_="${2}"
    local log_format_="[Common] [${cur_date_}] [${log_type_}]: ${msg_}"
    if [ "${log_type_}" = "INFO" ]; then
        echo "${log_format_}"
    elif [ "${log_type_}" = "WARNING" ]; then
        echo "${log_format_}"
    elif [ "${log_type_}" = "ERROR" ]; then
        echo "${log_format_}"
    elif [ "${log_type_}" = "DEBUG" ]; then
        echo "${log_format_}" 1> /dev/null
    fi
}

# 元素是否在列表中
__item_in_list() {
    local item="$1"
    shift
    local list_item
    local matched="false"

    for list_item in $*; do
        if [ "${item}" = "${list_item}" ]; then
            matched="true"
            break
        fi
    done
    echo "${matched}"
}

__pkg_in_pkgs() {
    __item_in_list "$@"
}

__block_in_blocks() {
    __item_in_list "$@"
}

# 检查路径是否为绝对路径
__check_abs_path() {
    local path="$1"

    if [ "${path#/}" != "${path}" ]; then
        is_abs_path="true"
    else
        is_abs_path="false"
    fi
}

__set_abs_path() {
    local install_path="$1"
    local path="$2"
    local varname="$3"
    local is_abs_path

    __check_abs_path "${path}"
    if [ "${is_abs_path}" != "true" ]; then
        eval "${varname}=\"${install_path}/${path}\""
    else
        eval "${varname}=\"${path}\""
    fi
}

# 从列表中移除一项
__remove_item_in_list() {
    local to_removed="$1"
    shift
    local list="$*"
    local list_item
    local new_list

    for list_item in ${list}; do
        if [ "${to_removed}" != "${list_item}" ]; then
            if [ "${new_list}" = "" ]; then
                new_list="${list_item}"
            else
                new_list="${new_list} ${list_item}"
            fi
        fi
    done
    echo "${new_list}"
}

# 包列表中移除包
__remove_pkg_in_pkgs() {
    __remove_item_in_list "$@"
}

# 返回列表长度
__length_list() {
    local list="$1"
    local var="$2"
    local list_item
    local cnt=0

    for list_item in ${list}; do
        cnt=$((cnt+1))
    done

    eval "${var}=\"${cnt}\""
}

# 获取列表索引值
__index_list() {
    local list="$1"
    shift
    local idx="$1"
    local var="$2"
    local list_item
    local cnt=0

    if [ $# -eq 0 ]; then
        return 0
    fi

    for list_item in ${list}; do
        if [ ${1} -eq ${cnt} ]; then
            eval "${2}=\"${list_item}\""
            shift 2
            if [ $# -eq 0 ]; then
                return 0
            fi
        fi
        cnt=$((cnt+1))
    done
}

# 冒泡排序，O(n**2)
__sort_blocks() {
    local blocks="$*"
    local block
    local sorted_blocks=""
    local max
    local i=0

    __length_list "${blocks}" "len_blocks"

    while [ "$i" -lt ${len_blocks} ]; do
        max=""
        for block in ${blocks}; do
            if [ "${block}" \> "${max}" ]; then
                max="${block}"
            fi
        done
        if [ "${sorted_blocks}" = "" ]; then
            sorted_blocks="${max}"
        else
            sorted_blocks="${max} ${sorted_blocks}"
        fi
        blocks="$(__remove_item_in_list "${max}" "${blocks}")"
        i=$((i+1))
    done

    echo "${sorted_blocks}"
}

__unpack_block_item() {
    local line="$1"
    echo "${line}" | tr '|' ' '
}

__pack_block_item() {
    local block_name="$1"
    local block_pkgs_str="$2"
    echo "${block_name}|${block_pkgs_str}"
}

__unpack_block_pkgs() {
    local block_pkgs_str="$1"
    echo "${block_pkgs_str}" | tr ',' ' '
}

__pack_block_pkgs() {
    echo "$*" | tr ' ' ','
}

__unpack_pkg_inner_softlink() {
    local pkg_inner_softlink_str="$1"
    local varname="$2"
    local item
    local temp
    OLD_IFS="${IFS}"
    IFS=";"
    temp=""
    for item in ${pkg_inner_softlink_str}; do
        if [ "${temp}" = "" ]; then
            temp="${item}"
        else
            temp="${temp} ${item}"
        fi
    done
    IFS="${OLD_IFS}"

    eval "${varname}=\"${temp}\""
}

# 移除字符串右侧的\r
__rstrip_like() {
    tr -d '\r'
}

check_install_path() {
    local install_path="$1"

    if [ "${install_path}" = "" ]; then
        log "ERROR" "install_path is empty!"
        exit 1
    fi
}

# 添加pkg对应的block_info
add_pkg_blocks_info() {
    local install_path="$1"
    local pkg="$2"
    shift; shift
    # blocks为待加入db.info的块列表
    local blocks="$*"
    local block_idx_value

    local db_filepath="${install_path}/${PKG_DB_INFO_RELPATH}"
    local db_dirpath="$(dirname "${db_filepath}")"

    local db_filepath_new="${install_path}/${PKG_DB_INFO_RELPATH}~"

    local block_list
    local block_name
    local block_pkgs_str
    local block_pkgs
    local len_block_pkgs
    local block_idx=0
    local len_blocks

    __length_list "${blocks}" "len_blocks"
    if [ ${len_blocks} -eq 0 ]; then
        return 0
    fi

    blocks="$(__sort_blocks "${blocks}")"

    if [ ! -d "${db_dirpath}" ]; then
        mkdir -p "${db_dirpath}"
        if [ $? -ne 0 ]; then
            log "ERROR" "mkdir ${db_dirpath} failed!"
            exit 1
        fi
    fi

    if [ ! -f "${db_filepath}" ]; then
        touch "${db_filepath}"
        if [ $? -ne 0 ]; then
            log "ERROR" "touch ${db_filepath} failed!"
            exit 1
        fi
        change_mod_and_own "${db_filepath}" "${DB_INFO_MOD}" "\$username:\$usergroup" "${INSTALL_FOR_ALL}"
    fi

    rm -f "${db_filepath_new}"
    if [ $? -ne 0 ]; then
        log "ERROR" "delete ${db_filepath_new} failed!"
        exit 1
    fi

    while read line; do
        block_list="$(__unpack_block_item "${line}")"
        __index_list "${block_list}" 0 "block_name"
        __index_list "${block_list}" 1 "block_pkgs_str"

        # 如果pkgs为空，则忽略该条目
        if [ "${block_pkgs_str}" = "" ]; then
            continue
        fi

        block_pkgs="$(__unpack_block_pkgs "${block_pkgs_str}")"
        __length_list "${block_pkgs}" "len_block_pkgs"
        if [ ${len_block_pkgs} -eq 0 ]; then
            continue
        fi

        if [ ${block_idx} -lt ${len_blocks} ]; then
            __index_list "${blocks}" ${block_idx} "block_idx_value"
        fi

        # blocks中的块名小于db中的当前块名，直接添加到db中。
        # 注意不能continue，因为还要处理db的当前条目。
        while [ ${block_idx} -lt ${len_blocks} ] && [ "${block_idx_value}" \< "${block_name}" ]; do
            __pack_block_item "${block_idx_value}" "${pkg}" >> "${db_filepath_new}"
            if [ $? -ne 0 ]; then
                log "ERROR" "write ${db_filepath_new} failed!"
                exit 1
            fi
            block_idx=$((block_idx+1))
            __index_list "${blocks}" ${block_idx} "block_idx_value"
        done

        # 已经没有需要添加的块，保持db的当前条目。
        if [ ${block_idx} -eq ${len_blocks} ]; then
            __pack_block_item "${block_name}" "${block_pkgs_str}" >> "${db_filepath_new}"
            if [ $? -ne 0 ]; then
                log "ERROR" "write ${db_filepath_new} failed!"
                exit 1
            fi
            continue
        fi

        # blocks中的块名大于db中的当前块名，保持db的当前条目。
        if [ "${block_idx_value}" \> "${block_name}" ]; then
            __pack_block_item "${block_name}" "${block_pkgs_str}" >> "${db_filepath_new}"
            if [ $? -ne 0 ]; then
                log "ERROR" "write ${db_filepath_new} failed!"
                exit 1
            fi
            continue
        fi

        # blocks中的块名等于db中的当前块名，将pkg添加到当前条目pkg列表中。
        in_pkgs="$(__pkg_in_pkgs "${pkg}" "${block_pkgs}")"
        if [ "${in_pkgs}" = "true" ]; then
            __pack_block_item "${block_name}" "${block_pkgs_str}" >> "${db_filepath_new}"
            if [ $? -ne 0 ]; then
                log "ERROR" "write ${db_filepath_new} failed!"
                exit 1
            fi
        else
            __pack_block_item "${block_name}" "${block_pkgs_str},${pkg}" >> "${db_filepath_new}"
            if [ $? -ne 0 ]; then
                log "ERROR" "write ${db_filepath_new} failed!"
                exit 1
            fi
        fi
        block_idx=$((block_idx+1))
    done < "${db_filepath}"

    # blocks中剩余的块添加到db中。
    while [ ${block_idx} -lt ${len_blocks} ]; do
        __index_list "${blocks}" ${block_idx} "block_idx_value"
        echo "${block_idx_value}|${pkg}" >> "${db_filepath_new}"
        block_idx=$((block_idx+1))
    done

    cp "${db_filepath_new}" "${db_filepath}"
    if [ $? -ne 0 ]; then
        log "ERROR" "replace ${db_filepath} failed!"
        exit 1
    fi

    rm -f "${db_filepath_new}"
    if [ $? -ne 0 ]; then
        log "ERROR" "delete ${db_filepath_new} failed!"
        exit 1
    fi
}

# 删除block_info中的pkg信息
del_blocks_info_pkg() {
    local install_path="$1"
    local pkg="$2"

    local db_filepath="${install_path}/${PKG_DB_INFO_RELPATH}"
    local db_dirpath="$(dirname "${db_filepath}")"

    local db_filepath_new="${install_path}/${PKG_DB_INFO_RELPATH}~"

    local block_list
    local block_name
    local block_pkgs_str
    local block_pkgs
    local len_block_pkgs

    if [ ! -f "${db_filepath}" ]; then
        return 0
    fi

    rm -f "${db_filepath_new}"
    if [ $? -ne 0 ]; then
        log "ERROR" "delete ${db_filepath_new} failed!"
        exit 1
    fi

    while read line; do
        block_list="$(__unpack_block_item "${line}")"

        __index_list "${block_list}" 0 "block_name"
        __index_list "${block_list}" 1 "block_pkgs_str"

        block_pkgs="$(__unpack_block_pkgs "${block_pkgs_str}")"

        block_pkgs="$(__remove_pkg_in_pkgs "${pkg}" "${block_pkgs}")"

        __length_list "${block_pkgs}" "len_block_pkgs"

        if [ ${len_block_pkgs} -gt 0 ]; then
            block_pkgs_str="$(__pack_block_pkgs "${block_pkgs}")"
            __pack_block_item "${block_name}" "${block_pkgs_str}" >> "${db_filepath_new}"
            if [ $? -ne 0 ]; then
                log "ERROR" "write ${db_filepath_new} failed!"
                exit 1
            fi
        fi
        # 当前条目如果没有pkg使用，则删除。
    done < "${db_filepath}"

    if [ ! -f "${db_filepath_new}" ]; then
        rm -f "${db_filepath}"
        if [ $? -ne 0 ]; then
            log "ERROR" "delete ${db_filepath} failed!"
            exit 1
        fi
    else
        mv -f "${db_filepath_new}" "${db_filepath}"
        if [ $? -ne 0 ]; then
            log "ERROR" "replace ${db_filepath} failed!"
            exit 1
        fi
    fi
}

# 获取所有的blocks
get_blocks_info() {
    local install_path="$1"

    local db_filepath="${install_path}/${PKG_DB_INFO_RELPATH}"

    local block_list
    local block_name
    local blocks=""

    if [ ! -f "${db_filepath}" ]; then
        return 0
    fi

    while read line; do
        block_list="$(__unpack_block_item "${line}")"

        __index_list "${block_list}" 0 "block_name"
        if [ "${blocks}" = "" ]; then
            blocks="${block_name}"
        else
            blocks="${blocks} ${block_name}"
        fi
    done < "${db_filepath}"

    echo "${blocks}"
}

# 解析filelist.csv脚本
parse_filelist() {
    local install_type="$1"
    local install_path="$2"
    local operate_type="$3"
    local filelist_path="$4"
    local feature_type="$5"
    local filter_type="$6"
    local install_type_list
    local operate_type_list
    local feature_type_list

    local blocks="$(get_blocks_info "${install_path}")"

    if [ "$install_type" != "full" ]; then
        install_type_list="all $install_type"
    else
        install_type_list="all docker devel run"
    fi
    if [ "$feature_type" != "all" ]; then
        feature_list="$(echo $feature_type | tr ',' ' ')"
        feature_type_list="all ${feature_list}"
    else
        feature_type_list="all"
    fi
    operate_type_list="$operate_type"

    if [ ! -f "$filelist_path" ]; then
        log "ERROR" "filelist $filelist_path does not exist!"
        exit 1
    fi
    # filelist中的feature为all，表示所有场景下都安装这个文件
    # 输入参数中的feature为all，表示安装所有的feature
    # 输入参数中的feature如果不为all，则安装filelist中的feature为all的文件，以及feature匹配的文件
    filelist=$(awk -v install_type_list="${install_type_list}" -v operate_type_list="${operate_type_list}" -v feature_type_list="${feature_type_list}" -v filter_type="${filter_type}" -v blocks="${blocks}" '
        BEGIN{
            FS= ","
            split(blocks, blocks_list, " ")
            split(feature_type_list, input_feature_type_arr, " ")
        }
        {
            if (operate_type_list !~ $2) next

            matched_install_type=0
            split($8, install_types, ";");
            for (i in install_types){
                if(install_type_list  ~  install_types[i]){
                    matched_install_type=1;
                    break;
                }
            }
            if (matched_install_type==0) next

            if ((filter_type == "filter_by_pkg_inner_softlink") && ($15 == "NA")) next

            if (filter_type == "filter_by_blocks") {
                matched_blocks=0
                for (i in blocks_list) {
                    if (blocks_list[i] == $14) {
                        matched_blocks=1
                        break;
                    }
                }
                if (matched_blocks==1) next
            }

            matched_feature_type=1
            if($10 != "all" && feature_type_list != "all" )
            {
                matched_feature_type=0
                split($10, features, ";")
                for(i in features)
                {
                    for(j in input_feature_type_arr) {
                        if(input_feature_type_arr[j] == features[i]) {
                            matched_feature_type=1
                            break;
                        }
                    }
                }
            }

            if (matched_feature_type==0) next

            matched_dependency=1;
            if($11 != "NA" && feature_type_list != "all" )
            {
                split($11, dependency, ";")
                for(f in dependency)
                {
                    if(feature_type_list ~ dependency[f])
                    {
                        continue;
                    }
                    else
                    {
                        matched_dependency=0;
                        break;
                    }
                }
            }
            if (matched_dependency==0) next

            print $3,$4,$6,$7,$9,$12,$13,$14,$15
        }' "$filelist_path")
}

# 迭代遍历filelist中的条目，执行一些操作
foreach_filelist() {
    local filter_type="$1"
    local exec_func="$2"
    local install_type="$3"
    local install_path="$4"
    local operate_type="$5"
    local filelist_path="$6"
    local feature_type="$7"
    local sort_filelist="$8"  # 排序filelist，可选参数为[no,reverse]
    local exec_mode="$9"  # 执行模式，可选参数为[normal,concurrency]
    local array
    local len_array
    local i
    local ret

    parse_filelist "${install_type}" "${install_path}" "${operate_type}" "${filelist_path}" "${feature_type}" "${filter_type}"

    if [ "${sort_filelist}" = "reverse" ]; then
        # 对第二列文件路径做倒序排列，保证先删除子文件夹，再删除父文件夹
        filelist=$(echo "$filelist" | sort -k2 -r)
    elif [ "${sort_filelist}" = "no" ]; then
        # 不排序
        echo > /dev/null
    else
        log "ERROR" "sort_filelist param wrong! sort_filelist is ${sort_filelist}, exec_func is ${exec_func}, parse_func is ${parse_func}"
        exit 1
    fi

    if [ "${exec_mode}" = "concurrency" ] || [ "${exec_mode}" = "normal" ]; then
        # 并发模式或正常模式
        echo > /dev/null
    else
        log "ERROR" "exec_mode param wrong! exec_mode is ${exec_mode}, exec_func is ${exec_func}, parse_func is ${parse_func}"
        exit 1
    fi

    while read line; do
        array=${line}
        __length_list "${array}" "len_array"
        if [ ${len_array} -eq 0 ]; then
            continue
        fi
        "${exec_func}" "${install_path}" "${line}"
    done << EOF
${filelist}
EOF

    if [ "${exec_mode}" = "concurrency" ]; then
        wait
    fi
}

# 解析stash_mod.txt
parse_stashmod() {
    local stashmod_path="$1"

    if [ ! -f "${stashmod_path}" ]; then
        log "ERROR" "stash mod ${stashmod_path} does not exist!"
        exit 1
    fi

    stashmod_list=$(awk '
        BEGIN{
            FS=":"
        }
        {
            print $1,$2
        }' "${stashmod_path}")
}

# 迭代遍历stash_mod中的条目，执行操作
foreach_stashmod() {
    local exec_func="$1"
    local install_path="$2"
    local sort_type="$3"  # 排序方式，可选参数为[no,reverse]
    local array
    local len_array
    local line

    parse_stashmod "${install_path}/${STASH_MOD_PATH}"

    if [ "${sort_type}" = "reverse" ]; then
        # 对第二列文件路径做倒序排列，保证先处理子文件夹，再处理父文件夹
        stashmod_list=$(echo "${stashmod_list}" | sort -k1 -r)
    elif [ "${sort_type}" = "no" ]; then
        # 不排序
        echo > /dev/null
    else
        log "ERROR" "sort_type param wrong! sort_type is ${sort_type}, exec_func is ${exec_func}"
        exit 1
    fi

    while read line; do
        array=${line}
        __length_list "${array}" "len_array"
        if [ ${len_array} -eq 0 ]; then
            continue
        fi
        "${exec_func}" "${install_path}" "${line}"
    done << EOF
${stashmod_list}
EOF
}

# 修改各文件及目录的权限
change_mod() {
    local mod="$2"
    local install_for_all="$3"
    # 对于软连接，可能目标文件还没有拷贝进来，导致无法修改mod，这里过滤掉软连接
    if [ -L "$1" ]; then
        return 0
    fi
    if [ "$2" != "NA" ]; then
        # 如果设置了install_for_all，则安装时other权限跟group权限对齐
        if [ "${install_for_all}" = "y" ]; then
            local len_mod=${#mod}
            local new_mod="${mod%?}"
            local new_mod="${new_mod}${new_mod#${new_mod%?}}"
            chmod "${new_mod}" "$1"
        else
            chmod "$2" "$1"
        fi
        if [ $? -ne 0 ]; then
            log "ERROR" "$1 chmod failed!"
            exit 1
        fi
    fi
}

# 修改各文件及目录的属性
change_own() {
    if [ "$2" != "NA" ]; then
        if [ "$2" = "\$username:\$usergroup" ]; then
            chown -h "${USERNAME}:${USERGROUP}" "$1"
        else
            chown -h "$2" "$1"
        fi
        if [ $? -ne 0 ]; then
            log "ERROR" "$1 chown failed!"
            exit 1
        fi
    fi
}

# 准备并且检查软链接路径
prepare_and_check_softlink_path() {
    local softlink_abs="$1"
    local softlink_dir="$(dirname "${softlink_abs}")"

    if [ ! -d "${softlink_dir}" ]; then
        mkdir -p "${softlink_dir}"
    fi

    # 如果目标路径是个软链接，则移除
    if [ -L "${softlink_abs}" ]; then
        rm -f "${softlink_abs}"
        if [ $? -ne 0 ]; then
            log "ERROR" "remove softlink ${softlink_abs} failed! (create relative softlink)"
            exit 1
        fi
    fi

    # 不允许目标路径已经是一个目录，防止软链接到错误的位置。
    if [ -d "${softlink_abs}" ]; then
        log "ERROR" "softlink existed dir ${softlink_abs}!"
        exit 1
    fi
}

# 使用绝对路径创建软连接
create_abs_softlink() {
    local softlink_abs="$2"
    local link_target

    prepare_and_check_softlink_path "${softlink_abs}"

    link_target=$(
        cd "$(dirname "$1")" || return
        pwd
    )/$(basename "$1")

    ln -sf "${link_target}" "$2"
    if [ $? -ne 0 ]; then
        log "ERROR" "$2 softlink to ${link_target} failed!"
        exit 1
    fi
}

# 使用相对路径创建软连接
create_rel_softlink() {
    local install_path="$1"
    local target="$2"
    local softlink="$3"
    local softlink_abs
    local softlink_dir
    local softlink_name
    local softlink_rel

    softlink_abs="${install_path}/${softlink}"
    softlink_dir="$(dirname "${softlink_abs}")"
    softlink_name="$(basename "${softlink_abs}")"

    prepare_and_check_softlink_path "${softlink_abs}"

    softlink_rel=$(echo | awk -v target="${target}" -v softlink="${softlink}" '
        BEGIN {
            len_t = split(target, target_list, "/")
            len_s = split(softlink, softlink_list, "/")

            if (len_t < len_s) {
                len_min = len_t
            } else {
                len_min = len_s
            }

            i = 1
            while (i <= len_min && target_list[i] == softlink_list[i]) {
                ++i
            }
            --i

            uplevel = len_s - i - 1
            rel_prefix = ""
            if (uplevel > 0) {
                for (j = 0; j < uplevel; ++j) {
                    rel_prefix = rel_prefix "../"
                }
            }

            rel_suffix = target_list[i+1]
            for (j = i+2; j <= len_t; ++j) {
                rel_suffix = rel_suffix "/" target_list[j]
            }
            print rel_prefix rel_suffix
        }
    ')

    cd "${softlink_dir}"

    ln -sf "${softlink_rel}" "${softlink_name}"
    if [ $? -ne 0 ]; then
        cd - > /dev/null
        log "ERROR" "${softlink} softlink to ${target} failed!"
        exit 1
    fi

    cd - > /dev/null
}

# 创建软连接
create_softlink() {
    local install_path="$1"
    local target="$2"
    local softlink="$3"
    local target_abs
    local softlink_abs
    local is_abs_path

    if [ "${softlink}" = "NA" ]; then
        return 0
    fi

    __check_abs_path "${target}"
    if [ "${is_abs_path}" = "true" ]; then
        __set_abs_path "${install_path}" "${softlink}" "softlink_abs"
        create_abs_softlink "${target}" "${softlink_abs}"
        return 0
    fi

    __check_abs_path "${softlink}"
    if [ "${is_abs_path}" = "true" ]; then
        __set_abs_path "${install_path}" "${target}" "target_abs"
        create_abs_softlink "${target_abs}" "${softlink}"
        return 0
    fi

    create_rel_softlink "${install_path}" "${target}" "${softlink}"
}

# 创建stash_mod.txt文件
create_stash_mod() {
    local install_path="$1"
    rm -f "${install_path}/${STASH_MOD_PATH}"
    touch "${install_path}/${STASH_MOD_PATH}"
    chmod ${STASH_FILE_MOD} "${install_path}/${STASH_MOD_PATH}"
}

# 删除stash_mod.txt文件
remove_stash_mod() {
    local install_path="$1"
    rm -f "${install_path}/${STASH_MOD_PATH}"
}

# 获取文件权限
get_file_mod() {
    local path="$1"
    stat -c %a "${path}"
}

# 创建文件夹
create_folder() {
    local install_path="$1"
    local target="$2"
    local softlink="$3"
    local target_abs

    __set_abs_path "${install_path}" "${target}" "target_abs"

    if [ ! -d "${target_abs}" ]; then
        mkdir -p "${target_abs}"
        if [ $? -ne 0 ]; then
            log "ERROR" "${target_abs} mkdir failed!"
            exit 1
        fi
        change_mod "${target_abs}" "${RESET_MOD}" ""
    fi
    if [ "${softlink}" != "NA" ]; then
        create_softlink "${install_path}" "${target}" "${softlink}"
    fi
}

# 创建目录
create_dirs() {
    local install_path="$1"
    local line="$2"
    local target
    local target_abs
    local softlink
    local is_abs_path

    __index_list "${line}" 1 "target" 4 "softlink"

    __set_abs_path "${install_path}" "${target}" "target_abs"

    if [ -L "${target_abs}" ] ; then
        rm -f "${target_abs}"
        log "WARNING" "${target_abs} is an existing soft-link, deleted."
    fi
    create_folder "${install_path}" "${target}" "${softlink}"
}

# 修改目录的权限和属组
reset_mod_dirs() {
    local install_path="$1"
    local line="$2"
    local mod
    local target
    local target_abs
    local is_abs_path

    __index_list "${line}" 1 "target"

    __set_abs_path "${install_path}" "${target}" "target_abs"

    # 目录不存在时跳过
    if [ ! -d "${target_abs}" ]; then
        return 0
    fi
    # 只处理目录，没有处理目录的软链接
    change_mod "${target_abs}" "${RESET_MOD}" ""
}

# 修改目录的权限和属组，暂存原文件权限
reset_mod_dirs_with_stash_mod() {
    local install_path="$1"
    local line="$2"
    local mod
    local target
    local target_abs
    local is_abs_path

    __index_list "${line}" 1 "target"

    __set_abs_path "${install_path}" "${target}" "target_abs"

    # 目录不存在时跳过
    if [ ! -d "${target_abs}" ]; then
        return 0
    fi

    mod="$(get_file_mod "${target_abs}")"
    echo "${target}:${mod}" >> "${install_path}/${STASH_MOD_PATH}"

    # 只处理目录，没有处理目录的软链接
    change_mod "${target_abs}" "${RESET_MOD}" ""
}

# 修改目录的权限和属组
restore_stash_mod() {
    local install_path="$1"
    local line="$2"
    local target
    local target_abs
    local mod
    local is_abs_path

    __index_list "${line}" 0 "target" 1 "mod"

    __set_abs_path "${install_path}" "${target}" "target_abs"

    # 目录不存在时跳过
    if [ ! -d "${target_abs}" ]; then
        return 0
    fi
    # 只处理目录，没有处理目录的软链接
    change_mod "${target_abs}" "${mod}" "${INSTALL_FOR_ALL}"
}

# 根据pkg_inner_softlink创建软链接
create_pkg_inner_softlink() {
    local install_path="$1"
    local line="$2"
    local array
    local target
    local pkg_inner_softlink
    local pkg_inner_softlink_list
    local is_abs_path

    __index_list "${line}" 1 "target" 8 "pkg_inner_softlink"
    # 移除行尾的\r
    pkg_inner_softlink="$(echo "${pkg_inner_softlink}" | __rstrip_like)"

    if [ "${pkg_inner_softlink}" = "NA" ]; then
        return 0
    fi

    __unpack_pkg_inner_softlink "${pkg_inner_softlink}" "pkg_inner_softlink_list"

    for pkg_inner_softlink_item in ${pkg_inner_softlink_list}; do
        create_softlink "${install_path}" "${target}" "${pkg_inner_softlink_item}"
    done
}

# 处理软链路径为已存在目录的情况
# minirc场景，install_path目录下，存在include目录
# 并且filelist.csv中配置了(x86_64|aarch64)-linux/include目录
# 则将(x86_64|aarch64)-linux/include中的内容，软链接至include目录下
deal_with_existed_dir() {
    local install_path="$1"
    local folder="$2"
    local filelist_path="$3"
    local install_path_regex="^(x86_64|aarch64)-linux/${folder}$"
    local install_sub_path_regex="^(x86_64|aarch64)-linux/${folder}/[^/]+$"

    if [ -L "${install_path}/${folder}" ] || [ ! -d "${install_path}/${folder}" ]; then
        return 0
    fi

    target_path=$(awk -v folder="${folder}" -v install_path_regex="${install_path_regex}" '
        BEGIN{
            FS= ","
        }
        {
            if ($2 != "mkdir") next

            if ($4 !~ install_path_regex) next

            if ($9 != folder) next

            print $4
        }' "${filelist_path}")

    if [ "${target_path}" = "" ]; then
        return 0
    fi

    awk -v folder="${folder}" -v install_path_regex="${install_path_regex}" -v install_sub_path_regex="${install_sub_path_regex}" '
        BEGIN{
            FS = ","; OFS = ","
        }
        {
            if ($2 != "mkdir" && $2 != "copy") {
                print $0
                next
            }
            # unset (x86_64|aarch64)-linux/<folder> softlink
            if ($4 ~ install_path_regex) {
                print $1, $2, $3, $4, $5, $6, $7, $8, "NA", $10, $11, $12, $13, $14, $15
                next
            }
            if ($4 !~ install_sub_path_regex) {
                print $0
                next
            }
            softlink = $9
            if (softlink != "NA") {
                print $0
                next
            }

            # set (x86_64|aarch64)-linux/<folder>/* softlink
            z = split($4, filepath_list, "/")
            softlink = folder "/" filepath_list[z]
            print $1, $2, $3, $4, $5, $6, $7, $8, softlink, $10, $11, $12, $13, $14, $15
        }' "${filelist_path}" > "${filelist_path}.tmp"
    if [ $? -ne 0 ]; then
        log "ERROR" "modify filelist for ${install_path}/${folder} failed!"
        exit 1
    fi

    mv -f "${filelist_path}.tmp" "${filelist_path}"
    if [ $? -ne 0 ]; then
        log "ERROR" "replace filelist for ${install_path}/${folder} failed!"
        exit 1
    fi
}

#执行创建目录动作
do_create_dirs() {
    local action="$1"
    local install_type="$2"
    local install_path="$3"
    local filelist_path="$4"
    local package="$5"
    local feature_type="$6"

    if [ "${action}" = "resetmod" ]; then
        if [ "${package}" != "" ]; then
            del_blocks_info_pkg "${install_path}" "${package}"
            create_stash_mod "${install_path}"
            foreach_filelist "NA" "reset_mod_dirs_with_stash_mod" "${install_type}" "${install_path}" "mkdir" "${filelist_path}" "${feature_type}" "no" "normal"
        else
            foreach_filelist "NA" "reset_mod_dirs" "${install_type}" "${install_path}" "mkdir" "${filelist_path}" "${feature_type}" "no" "normal"
        fi
    elif [ "${action}" = "all" ] || [ "${action}" = "mkdir" ]; then
        deal_with_existed_dir "$install_path" "include" "$filelist_path"
        deal_with_existed_dir "$install_path" "lib64" "$filelist_path"
        # 先重置目录权限配置，防止软链接时缺少权限
        foreach_filelist "NA" "reset_mod_dirs" "$install_type" "$install_path" "mkdir" "$filelist_path" "$feature_type" "no" "normal"
        foreach_filelist "NA" "create_dirs" "$install_type" "$install_path" "mkdir" "$filelist_path" "$feature_type"  "no" "normal"
        foreach_filelist "filter_by_pkg_inner_softlink" "create_pkg_inner_softlink" "$install_type" "$install_path" "mkdir" "$filelist_path" "$feature_type" "no" "normal"
    else
        log "ERROR" "action wrong! action is ${action}"
        exit 1
    fi
}

# 拷贝文件
copy_file() {
    local install_path="$1"
    local source="$2"
    local target="$3"
    local softlink="$4"
    local target_abs
    local target_dir

    __set_abs_path "${install_path}" "${target}" "target_abs"
    target_dir="$(dirname "${target_abs}")"

    if [ -e "${source}" ]; then
        cp -rf "${source}" "${target_dir}"
        if [ $? -ne 0 ]; then
            log "ERROR" "${source} copy failed!"
            exit 1
        fi
        if [ "${softlink}" != "NA" ]; then
            create_softlink "${install_path}" "${target}" "${softlink}"
        fi
    else
        log "ERROR" "copy file ${source} does not exist!"
        exit 1
    fi
}

# 复制文件
copy_files() {
    local install_path="$1"
    local line="$2"
    local src
    local target
    local target_abs
    local softlink
    local configurable
    local hash_value
    local tmpdir
    local is_abs_path

    __index_list "${line}" 0 "src" 1 "target" 4 "softlink" 5 "configurable" 6 "hash_value"

    __set_abs_path "${install_path}" "${target}" "target_abs"

    tmpdir="$(dirname "${target_abs}")"
    if [ ! -d "${tmpdir}" ]; then
        mkdir -p "${tmpdir}"
    fi

    # 如果目标文件已经存在，而且是配置文件，则不执行覆盖操作
    if [ -e "${target_abs}" ] && [ "${configurable}" = "TRUE" ]; then
        return 0
    fi

    copy_file "${install_path}" "${src}" "${target}" "${softlink}"
}

__set_total_uninstall_path() {
    local install_path="$1"
    local varname="$2"

    eval "${varname}=\"${install_path}/cann_uninstall.sh\""
}

# 是否为hilinux环境
__is_hilinux() {
    local varname="$1"

    which lsattr > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        eval "${varname}=\"true\""
    else
        eval "${varname}=\"false\""
    fi
}

# 移除文件上的不可修改权限
__remove_immutable() {
    local file="$1"
    local is_hilinux

    # 文件不存在则退出。
    if [ ! -f "${file}" ]; then
        return 0
    fi

    __is_hilinux "is_hilinux"
    if [ "${is_hilinux}" = "true" ]; then
        return 0
    fi

    attr="$(lsattr "${file}" | cut -d' ' -f1 | grep -o "i")"
    if [ "${attr}" != "" ]; then
        chattr -i "${file}"
    fi
}

# 创建总uninstall.sh脚本
__create_uninstall() {
    local file="$1"

    if [ ! -f "${file}" ]; then
        cat > ${file} <<EOF
#!/bin/bash

SHELL_DIR="\$(dirname "\${BASH_SOURCE[0]}")"
INSTALL_PATH="\$(cd "\${SHELL_DIR}" && pwd)"

uninstall_package() {
    local package="\$1"

    cd \${INSTALL_PATH}/\${package}/script
    ./uninstall.sh
}

EOF
    fi
}

# 添加uninstall.sh文件中的uninstall_package函数调用
__add_uninstall_package() {
    local file="$1"
    local package="$2"

    echo "uninstall_package \"${package}\"" >> "${file}"
}

# 删除uninstall.sh文件中的uninstall_package函数调用
__remove_uninstall_package() {
    local file="$1"
    local package="$2"

    if [ -f "${file}" ]; then
        sed -i "/uninstall_package \"${package}\"/d" "${file}"
        if [ $? -ne 0 ]; then
            log "ERROR" "remove ${file} uninstall_package command failed!"
            exit 1
        fi
    fi
}

# 删除uninstall.sh文件，如果已经没有uninstall_package调用
__remove_uninstall_file_if_no_content() {
    local file="$1"
    local num

    if [ ! -f "${file}" ]; then
        return 0
    fi

    # 注意uninstall_package后有一个空格，表示函数调用
    num=$(grep "uninstall_package " ${file} | wc -l)
    if [ ${num} -eq 0 ]; then
        rm -f "${file}" > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            log "WARNING" "Delete file:${file} failed, please delete it by yourself."
        fi
    fi
}

# 执行拷贝动作
do_copy_files() {
    local install_type="$1"
    local install_path="$2"
    local filelist_path="$3"
    local package="$4"
    local feature_type="$5"
    local total_uninstall_path

    foreach_filelist "NA" "copy_files" "$install_type" "$install_path" "copy" "$filelist_path" "$feature_type" "no" "normal"
    # pkg_inner_softlink处理
    foreach_filelist "filter_by_pkg_inner_softlink" "create_pkg_inner_softlink" "$install_type" "$install_path" "copy" "$filelist_path" "$feature_type" "no" "normal"

    if [ "${package}" != "" ] && [ "${SET_CANN_UNINSTALL}" = "y" ]; then
        __set_total_uninstall_path "${install_path}" "total_uninstall_path"
        if [ -f "${install_path}/${package}/script/uninstall.sh" ]; then
            __create_uninstall "${total_uninstall_path}"
            __remove_immutable "${total_uninstall_path}"
            change_own "${total_uninstall_path}" "${USERNAME}:${USERGROUP}"
            change_mod "${total_uninstall_path}" "${SETENV_WRITEABLE_MOD}" ""
            __add_uninstall_package "${total_uninstall_path}" "${package}"
            change_mod "${total_uninstall_path}" "${TOTAL_UNINSTALL_MOD}" "${INSTALL_FOR_ALL}"
        fi
    fi
}

# 修改权限和属组
change_mod_and_own(){
    local target="$1"
    local mod="$2"
    local own="$3"
    local install_for_all="$4"
    change_mod "${target}" "${mod}" "${install_for_all}"
    change_own "${target}" "${own}"
}

# 修改文件的权限和属组
change_mod_and_own_files() {
    local install_path="$1"
    local line="$2"
    local target
    local mod
    local own
    local is_abs_path

    __index_list "${line}" 1 "target" 2 "mod" 3 "own"
    __check_abs_path "${target}"
    if [ "${is_abs_path}" != "true" ]; then
        target="${install_path}/${target}"
    fi
    if [ -d "${target}" ]; then
        #echo "$target is directory,skip!"
        return 0
    fi
    # 只处理文件，没有处理文件的软链接
    change_mod_and_own "${target}" "${mod}" "${own}" "${INSTALL_FOR_ALL}" &
}

# 修改目录的权限和属组
change_mod_and_own_dirs() {
    local install_path="$1"
    local line="$2"
    local target
    local mod
    local own
    local is_abs_path

    __index_list "${line}" 1 "target" 2 "mod" 3 "own"

    __check_abs_path "${target}"
    if [ "${is_abs_path}" != "true" ]; then
        target="${install_path}/${target}"
    fi
    if [ ! -d "${target}" ]; then
        #echo "$target is directory,skip!"
        return 0
    fi
    # 只处理目录，没有处理目录的软链接
    change_mod_and_own "${target}" "${mod}" "${own}" "${INSTALL_FOR_ALL}"
}

# filelist中的使用到的blocks，添加到db.info中
add_filelist_blocks_info() {
    local install_type="$1"
    local install_path="$2"
    local filelist_path="$3"
    local feature_type="$4"
    local package="$5"
    local blocks=""

    parse_filelist "${install_type}" "${install_path}" "copy del mkdir" "${filelist_path}" "${feature_type}" "NA"
    filelist="$(echo "${filelist}" | cut -d' ' -f8 | sort | uniq)"

    while read line; do
        block_name="${line}"
        if [ "${block_name}" = "" ]; then
            continue
        fi
        if [ "${blocks}" = "" ]; then
            blocks="${block_name}"
        else
            blocks="${blocks} ${block_name}"
        fi
    done << EOF
${filelist}
EOF
    add_pkg_blocks_info "${install_path}" "${package}" "${blocks}"
}

# 修改文件和目录的权限
do_chmod_file_dir() {
    local install_type="$1"
    local install_path="$2"
    local filelist_path="$3"
    local feature_type="$4"
    local package="$5"

    foreach_filelist "NA" "change_mod_and_own_files" "$install_type" "$install_path" "copy del" "$filelist_path" "$feature_type" "no" "concurrency"
    foreach_filelist "NA" "change_mod_and_own_dirs" "$install_type" "$install_path" "mkdir" "$filelist_path" "$feature_type" "reverse" "normal"

    if [ "${package}" != "" ]; then
        add_filelist_blocks_info "$install_type" "$install_path" "$filelist_path" "$feature_type" "$package"
    fi
}

# 删除软连接
remove_softlink() {
    local softlink="$1"

    if [ "${softlink}" = "" ]; then
        return 0
    fi

    if [ "${softlink}" != "NA" ] && [ -L "${softlink}" ]; then
        rm -f "${softlink}"
        if [ $? -ne 0 ]; then
            log "ERROR" "${softlink} remove failed!"
            exit 1
        fi
    fi
}

#移除文件
remove_file() {
    local target="$1"
    local softlink="$2"
    if [ -e "${target}" ] || [ -L "${target}" ]; then
        rm -f "${target}"
        if [ $? -ne 0 ]; then
            log "ERROR" "${target} remove failed!"
            exit 1
        fi
    fi
    remove_softlink "${softlink}"
}

#移除文件夹
remove_dir() {
    if [ -e "$1" ] || [ -L "$1" ]; then
        rm -fr "$1"
        if [ $? -ne 0 ]; then
            log "ERROR" "$1 remove failed!"
            exit 1
        fi
    fi
}

# 删除安装生成的pkg_inner_softlinks
remove_install_pkg_inner_softlinks() {
    local install_path="$1"
    local line="$2"
    local target
    local pkg_inner_softlink
    local pkg_inner_softlink_list
    local pkg_inner_softlink_abs

    __index_list "${line}" 1 "target" 8 "pkg_inner_softlink"
    # 移除行尾的\r
    pkg_inner_softlink="$(echo "${pkg_inner_softlink}" | __rstrip_like)"
    if [ "${target}" != "NA" ]; then
        if [ "${pkg_inner_softlink}" != "NA" ]; then
            __unpack_pkg_inner_softlink "${pkg_inner_softlink}" "pkg_inner_softlink_list"

            for pkg_inner_softlink in ${pkg_inner_softlink_list}; do
                __set_abs_path "${install_path}" "${pkg_inner_softlink}" "pkg_inner_softlink_abs"
                remove_softlink "${pkg_inner_softlink_abs}"
            done
        fi
    fi
}

# 删除安装文件，入参为$1: install_type, $2: install_path $3:filelist_path
remove_install_files() {
    local install_path="$1"
    local line="$2"
    local target
    local softlink
    local configurable
    local hash_value
    local is_abs_path

    __index_list "${line}" 1 "target" 4 "softlink" 5 "configurable" 6 "hash_value"

    if [ "${target}" != "NA" ]; then
        __check_abs_path "${target}"
        if [ "${is_abs_path}" != "true" ]; then
            target="${install_path}/${target}"
        fi
        if [ -d "${target}" ] && [ ! -L "${target}" ]; then
            #echo "$target is directory,skip!"
            return 0
        fi
        if [ "${softlink}" != "NA" ]; then
            __check_abs_path "${softlink}"
            if [ "${is_abs_path}" != "true" ]; then
                softlink="${install_path}/${softlink}"
            fi
        fi
        # 配置文件不删除
        if [ "${configurable}" = "TRUE" ]; then
            echo "${hash_value} ${target}" | sha256sum --check > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                log "WARNING" "${target} has been modified!"
                return 0
            fi
        fi
        remove_file "${target}" "${softlink}" &
    fi
}

# 删除安装文件夹生成的软链接
remove_install_dirs_softlink() {
    local install_path="$1"
    local line="$2"
    local target
    local softlink
    local is_abs_path

    __index_list "${line}" 1 "target" 4 "softlink"

    if [ "$target" != "NA" ]; then
        if [ "${softlink}" != "NA" ]; then
            __check_abs_path "${softlink}"
            if [ "${is_abs_path}" != "true" ]; then
                softlink="${install_path}/${softlink}"
            fi
        fi

        remove_softlink "${softlink}"
    fi
}

# 删除安装文件夹
remove_install_dirs() {
    local install_path="$1"
    local line="$2"
    local target
    local is_abs_path

    __index_list "${line}" 1 "target"

    if [ "${target}" != "NA" ]; then
        __check_abs_path "${target}"
        if [ "${is_abs_path}" != "true" ]; then
            target="${install_path}/${target}"
        fi
        if [ ! -d "${target}" ]; then
            #echo "dir $target not exists!"
            return 0
        fi
        # 不同的blocks中，可能配置相同的dir_info。目录不为空时不删除。
        if [ "$(ls -A "${target}")" ]; then
            #echo "$target not empty,will not remove!"
            return 0
        fi
        #echo "delete $target"
        remove_dir "${target}"
    fi
}

# 删除安装文件与目录等
do_remove() {
    local install_type="$1"
    local install_path="$2"
    local filelist_path="$3"
    local package="$4"
    local feature_type="$5"
    local total_uninstall_path
    local oldmod
    local func_before_remove
    local func_after_remove_01
    local func_after_remove_02

    if [ "${package}" != "" ]; then
        # root帐户--uninstall时，存在不调用restoremod，直接调用remove的场景。（toolkit包）
        func_before_remove="del_blocks_info_pkg \"${install_path}\" \"${package}\""
        if [ -f "${install_path}/${STASH_MOD_PATH}" ]; then
            # 删除文件后，恢复目录权限配置
            func_after_remove_01="foreach_stashmod \"restore_stash_mod\" \"${install_path}\" \"reverse\""
            func_after_remove_02="remove_stash_mod \"${install_path}\""
        fi

        __set_total_uninstall_path "${install_path}" "total_uninstall_path"

        if [ -f "${total_uninstall_path}" ]; then
            __remove_immutable "${total_uninstall_path}"
            oldmod="$(get_file_mod "${total_uninstall_path}")"
            change_mod "${total_uninstall_path}" "${SETENV_WRITEABLE_MOD}" ""
            __remove_uninstall_package "${total_uninstall_path}" "${package}"
            change_mod "${total_uninstall_path}" "${oldmod}" ""
            __remove_uninstall_file_if_no_content "${total_uninstall_path}"
        fi
    fi

    eval "${func_before_remove}"

    # del元素不会出现pkg_inner_softlink
    foreach_filelist "filter_by_pkg_inner_softlink" "remove_install_pkg_inner_softlinks" "$install_type" "$install_path" "mkdir copy" "$filelist_path" "$feature_type" "no" "normal"
    foreach_filelist "filter_by_blocks" "remove_install_files" "$install_type" "$install_path" "copy del" "$filelist_path" "$feature_type" "no" "concurrency"
    foreach_filelist "filter_by_blocks" "remove_install_dirs_softlink" "$install_type" "$install_path" "mkdir" "$filelist_path" "$feature_type" "no" "normal"
    foreach_filelist "filter_by_blocks" "remove_install_dirs" "$install_type" "$install_path" "mkdir" "$filelist_path" "$feature_type" "reverse" "normal"

    eval "${func_after_remove_01}"
    eval "${func_after_remove_02}"
}

# 打印安装信息
print_install_content() {
    local install_type="$1"
    local install_path="$2"
    local operate_type="$3"
    local filelist_path="$4"
    local feature_type="$5"

    parse_filelist "$install_type" "$install_path" "$operate_type" "$filelist_path" "$feature_type" "filter_by_blocks"

    while read line; do
        echo $line
    done << EOF
$filelist
EOF
    exit 0
}

# 检查参数。
__check_param() {
    local name="$1"
    local value

    value="$(eval echo \${${name}})"

    if [ "${value}" = "" ]; then
        log "ERROR" "need set ${name} parameter!"
        exit 1
    fi
}

# 修改路径的own
__chown_path() {
    local path="$1"
    local username="$2"
    local usergroup="$3"

    if [ "${username}" != "" ] && [ "${usergroup}" != "" ]; then
        chown -h "${username}:${usergroup}" "${path}"
        if [ $? -ne 0 ]; then
            log "ERROR" "${path} chown failed!"
            exit 1
        fi
    fi
}

# 创建配置所在目录
__create_config_dir() {
    local dir="$1"
    local username="$2"
    local usergroup="$3"

    if [ ! -d "${dir}" ]; then
        mkdir -p "${dir}"
        __chown_path "${dir}" "${username}" "${usergroup}"
    fi
}

# 创建配置文件
__create_config_file() {
    local file="$1"
    local username="$2"
    local usergroup="$3"

    if [ ! -f "${file}" ]; then
        touch "${file}"
        __chown_path "${file}" "${username}" "${usergroup}"
    fi
}

# 创建setenv文件
__create_setenv_file() {
    local file="$1"
    local shell_type="$2"

    if [ ! -f "${file}" ]; then
        echo "#!/usr/bin/env ${shell_type}" > ${file}
    fi
}

# 删除setenv文件，如果已经没有setenv内容
__remove_setenv_file_if_no_content() {
    local file="$1"
    local shell_type="$2"
    local num

    if [ ! -f "${file}" ]; then
        return 0
    fi

    num=$(grep "setenv.${shell_type}" ${file} | wc -l)
    if [ ${num} -eq 0 ]; then
        rm -f "${file}" > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            log "WARNING" "Delete file:${file} failed, please delete it by yourself."
        fi
    fi
}

# 删除rcfile中的source <path>
__remove_path_regex() {
    local path_regex="$1"
    local rcfile="$2"

    if [ -f "${rcfile}" ]; then
        sed -i "/source ${path_regex}$/d" "${rcfile}"
        if [ $? -ne 0 ]; then
            log "ERROR" "remove ${rcfile} source command failed!"
            exit 1
        fi
    fi
}

# 设置用户家目录路径
__set_userpath() {
    local username="$1"
    local docker_root="$2"

    if [ -z "${docker_root}" ] || [ "${docker_root}" = "/" ]; then
        userpath="$(eval echo "~${username}")"
    else
        if [ "${username}" = "root" ]; then
            userpath="${docker_root}/root"
        else
            userpath="${docker_root}/home/${username}"
        fi
    fi
}

# 添加source setenv.[shell]脚本到rc文件中
add_env_rc() {
    local install_path="$1"
    local setenv_filepath="$2"
    local package="$3"
    local shell_type="$4"
    local setenv="$5"
    local username="$6"
    local usergroup="$7"
    local docker_root="$8"
    # setenv传入路径正则规则
    # 必须以/开头（绝对路径）
    # 必须以/${package}/bin/setenv.${shell_type}结尾
    local path_regex="\/\(.\+\/\)\?${package}\/bin\/setenv.${shell_type}"
    local path_suffix="${package}/bin/setenv.${shell_type}"
    local matched
    local userpath
    local config_path

    __check_param "package"
    __check_param "username"
    __check_param "usergroup"

    echo "${shell_type}" | grep -E "^(bash|fish|csh)$" > /dev/null
    if [ $? -ne 0 ]; then
        log "ERROR" "shell type ${shell_type} not support!"
        exit 1
    fi

    matched="$(echo "${setenv_filepath}" | sed -n "/^${path_regex}$/p")"
    if [ -z "${matched}" ]; then
        log "ERROR" "setenv filepath is illegal, should endswith ${path_suffix}"
        exit 1
    fi

    if [ "${setenv}" = "y" ]; then
        __set_userpath "${username}" "${docker_root}"

        if [ "${shell_type}" = "bash" ]; then
            config_path="${userpath}/.bashrc"
            __create_config_dir "${userpath}" "${username}" "${usergroup}"
            __create_config_file "${config_path}" "${username}" "${usergroup}"

            __remove_path_regex "${path_regex}" "${config_path}"
            echo "source ${setenv_filepath}" >> "${config_path}"
        elif [ "${shell_type}" = "fish" ]; then
            config_path="${userpath}/.config/fish/config.fish"
            __create_config_dir "${userpath}" "${username}" "${usergroup}"
            __create_config_dir "${userpath}/.config" "${username}" "${usergroup}"
            __create_config_dir "${userpath}/.config/fish" "${username}" "${usergroup}"
            __create_config_file "${config_path}" "${username}" "${usergroup}"

            __remove_path_regex "${path_regex}" "${config_path}"
            echo "source ${setenv_filepath}" >> "${config_path}"
        elif [ "${shell_type}" = "csh" ]; then
            config_path="${userpath}/.cshrc"
            __create_config_dir "${userpath}" "${username}" "${usergroup}"
            __create_config_file "${config_path}" "${username}" "${usergroup}"

            __remove_path_regex "${path_regex}" "${config_path}"
            echo "set argv=(\"${setenv_filepath}\"); source ${setenv_filepath}" >> "${config_path}"
        fi
    fi

    if [ ! -d "${install_path}/bin" ]; then
        # 如果bin目录不存在，则不处理
        return 0
    fi

    if [ "${shell_type}" = "bash" ]; then
        config_path="${install_path}/bin/setenv.bash"
        __create_setenv_file "${config_path}" "${shell_type}"
        change_own "${config_path}" "${username}:${usergroup}"
        change_mod "${config_path}" "${SETENV_WRITEABLE_MOD}" ""
        __remove_path_regex "${path_regex}" "${config_path}"
        echo "source ${setenv_filepath}" >> "${config_path}"
    elif [ "${shell_type}" = "fish" ]; then
        config_path="${install_path}/bin/setenv.fish"
        __create_setenv_file "${config_path}"
        change_own "${config_path}" "${username}:${usergroup}"
        change_mod "${config_path}" "${SETENV_WRITEABLE_MOD}" ""
        __remove_path_regex "${path_regex}" "${config_path}"
        echo "source ${setenv_filepath}" >> "${config_path}"
    elif [ "${shell_type}" = "csh" ]; then
        config_path="${install_path}/bin/setenv.csh"
        __create_setenv_file "${config_path}"
        change_own "${config_path}" "${username}:${usergroup}"
        change_mod "${config_path}" "${SETENV_WRITEABLE_MOD}" ""
        __remove_path_regex "${path_regex}" "${config_path}"
        echo "set argv=(\"${setenv_filepath}\"); source ${setenv_filepath}" >> "${config_path}"
    fi
    change_mod "${config_path}" "${SETENV_MOD}" "${INSTALL_FOR_ALL}"
}

# 从rc文件中删除source setenv.[shell]
del_env_rc() {
    local install_path="$1"
    local setenv_filepath="$2"
    local shell_type="$3"
    local username="$4"
    local docker_root="$5"
    local path_regex
    local userpath
    local config_path
    local oldmod

    __check_param "username"

    echo "${shell_type}" | grep -E "^(bash|fish|csh)$" > /dev/null
    if [ $? -ne 0 ]; then
        log "ERROR" "shell type ${shell_type} not support!"
        exit 1
    fi

    __set_userpath "${username}" "${docker_root}"

    # 将路径中的/转换为\/
    path_regex="$(echo "${setenv_filepath}" | sed "s#\/#\\\/#g")"

    if [ "${shell_type}" = "bash" ]; then
        __remove_path_regex "${path_regex}" "${userpath}/.bashrc"
    elif [ "${shell_type}" = "fish" ]; then
        __remove_path_regex "${path_regex}" "${userpath}/.config/fish/config.fish"
    elif [ "${shell_type}" = "csh" ]; then
        __remove_path_regex "${path_regex}" "${userpath}/.cshrc"
    fi

    if [ "${shell_type}" = "bash" ]; then
        config_path="${install_path}/bin/setenv.bash"
    elif [ "${shell_type}" = "fish" ]; then
        config_path="${install_path}/bin/setenv.fish"
    elif [ "${shell_type}" = "csh" ]; then
        config_path="${install_path}/bin/setenv.csh"
    fi

    if [ ! -f "${config_path}" ]; then
        # 如果文件不存在，则不处理
        return 0
    fi

    oldmod="$(get_file_mod "${config_path}")"
    change_mod "${config_path}" "${SETENV_WRITEABLE_MOD}" ""
    __remove_path_regex "${path_regex}" "${config_path}"
    change_mod "${config_path}" "${oldmod}" ""

    __remove_setenv_file_if_no_content "${config_path}" "${shell_type}"
}

# 包名是否存在于db.info
pkg_in_dbinfo() {
    local install_path="$1"
    local pkg="$2"

    local db_filepath="${install_path}/${PKG_DB_INFO_RELPATH}"

    if [ -z "${install_path}" ]; then
        echo "false"
        exit 1
    fi

    if [ -z "${pkg}" ]; then
        echo "false"
        exit 2
    fi

    if [ ! -f "${db_filepath}" ]; then
        echo "false"
        return 0
    fi

    awk -v input_pkg="${pkg}" '
        BEGIN {
            FS = "|"
            found = "false"
        }
        {
            split($2, pkg_list, ",")

            for (i in pkg_list) {
                if (input_pkg == pkg_list[i]) {
                    found = "true"
                    exit
                }
            }
        }
        END {
            print found
        }' "${db_filepath}"
}

# 标准化feature参数
# 输入的feature参数中，如果有all字段（表示安装所有feature）
# 则将feature参数重置为all，在后续流程中安装所有feature
normalize_feature() {
    local feature="$1"
    local varname="$2"

    case "${feature}" in
    all,*)
        feature="all"
        ;;
    *,all)
        feature="all"
        ;;
    *,all,*)
        feature="all"
        ;;
    esac

    eval "${varname}=\"${feature}\""
}

help_info() {
    echo "Usage:"
    echo ""
    echo "  $0 [param1] --package=<package> [ --username=<user> --usergroup=<group> ] install_type install_path filelist_path"
    echo "  param1 can be one of the following:"
    echo "    --help       | -h      : Print out this help message"
    echo "    --copy       | -c      : Copy the install content"
    echo "    --mkdir      | -m      : Create the install folder, and set dir right"
    echo "    --makedir    | -d      : Create the install folder, not set dir right"
    echo "    --chmoddir   | -o      : Set dir right"
    echo "    --restoremod | -e      : Restore dir right"
    echo "    --remove     | -r      : Remove the install content"
    echo "    --print      | -p      : print the install content and folder info"
    echo "    --package=<package>  : specify package"
    echo "    --username=<user>    : specify user"
    echo "    --usergroup=<group>  : specify group"
    echo "    --install_for_all      : Install for all user"
    echo ""
    echo "  $0 [param2] --package=<package> --username=<user> [ --usergroup=<group> --docker-root=<path> ] setenv_filepath shell_type"
    echo "  param2 can be one of the following:"
    echo "    --add-env-rc           : add source setenv to rcfile"
    echo "    --del-env-rc           : del source setenv from rcfile"
    echo ""
    echo "  $0 [param3] --package=<package> install_path"
    echo "    --pkg-in-dbinfo        : does pkg in ascend_package_db.info"
    exit 0
}

# env rc相关命令
env_rc_commands() {
    local install_path
    local setenv_filepath
    local shell_type

    case "${OPERATE_TYPE}" in
    "add-env-rc")
        install_path="$1"
        setenv_filepath="$2"
        shell_type="$3"
        add_env_rc "${install_path}" "${setenv_filepath}" "${PACKAGE}" "${shell_type}" "${SETENV}" "${USERNAME}" "${USERGROUP}" "${DOCKER_ROOT}"
        exit 0
        ;;
    "del-env-rc")
        install_path="$1"
        setenv_filepath="$2"
        shell_type="$3"
        del_env_rc "${install_path}" "${setenv_filepath}" "${shell_type}" "${USERNAME}" "${DOCKER_ROOT}"
        exit 0
        ;;
    esac
}

# db.info相关命令
dbinfo_commands() {
    local install_path

    case "${OPERATE_TYPE}" in
    "pkg-in-dbinfo")
        install_path="$1"
        pkg_in_dbinfo "${install_path}" "${PACKAGE}"
        exit 0
    esac
}

# 内部命令，不对外使用
inner_commands() {
    local install_path
    local pkg
    local blocks

    case "${OPERATE_TYPE}" in
    "add-pkg-blocks")
        install_path="$1"
        pkg="$2"
        shift; shift
        blocks="$*"
        check_install_path "$install_path"
        add_pkg_blocks_info "${install_path}" "${pkg}" "${blocks}"
        exit 0
        ;;
    "del-pkg-blocks")
        install_path="$1"
        pkg="$2"
        check_install_path "$install_path"
        del_blocks_info_pkg "${install_path}" "${pkg}"
        exit 0
        ;;
    "print-blocks")
        install_path="$1"
        check_install_path "$install_path"
        blocks="$(get_blocks_info ${install_path})"
        echo "${blocks}"
        exit 0
        ;;
    esac
}

# 正式命令
formal_commands() {
    local install_type="$1"
    local install_path="$2"
    local input_feature
    local tmp_root
    local tmp_filelist_path

    if [ $# = 3 ]; then
        input_feature="all"
    else
        normalize_feature "$4" "input_feature"
    fi
    case "${OPERATE_TYPE}" in
    "mkdir")
        do_create_dirs "all" "$1" "$2" "$3" "${PACKAGE}" "$input_feature"
        ;;
    "makedir")
        do_create_dirs "mkdir" "$1" "$2" "$3" "${PACKAGE}" "$input_feature"
        ;;
    "copy")
        do_copy_files "$1" "$2" "$3" "${PACKAGE}" "$input_feature"
        ;;
    "chmoddir")
        do_chmod_file_dir "$install_type" "$2" "$3" "$input_feature" "${PACKAGE}"
        ;;
    "restoremod")
        do_create_dirs "resetmod" "$1" "$2" "$3" "${PACKAGE}" "$input_feature"
        ;;
    "remove")
        filelist_path="$3"
        tmp_root="$HOME"
        if ! test -d "$tmp_root"; then
            tmp_root="$PWD"
        fi
        tmp_filelist_path=$(mktemp "$tmp_root/filelist_XXXXXX" || exit 1)
        cp -f "$filelist_path" "$tmp_filelist_path"
        do_remove "$install_type" "$2" "$tmp_filelist_path" "${PACKAGE}" "$input_feature"
        rm -f "$tmp_filelist_path"
        ;;
    "print")
        print_install_content "$1" "$2" "${PRINT_OPERATE_TYPE}" "$3" "$input_feature"
        ;;
    esac
}

# 正式命令并加锁
formal_commands_with_lock() {
    local install_path="$2"
    local lockfile="${install_path}/ascend.lock"

    if [ ! -d "${install_path}" ]; then
        # 兼容覆盖安装场景，包安装目录被删除情况。
        mkdir -p "${install_path}"
    fi

    (
        flock -n 9 || exit 1
        formal_commands "$@"
    ) 9> "${lockfile}"

    if [ $? -ne 0 ]; then
        log "ERROR" "get ${install_path} lockfile failed!"
        exit 1
    fi

    rm -f "${lockfile}"
}


# 全局变量
# change_mod_and_own_files与change_mod_and_own_dirs函数中会使用
INSTALL_FOR_ALL=""
SETENV=""
SET_CANN_UNINSTALL=""

OPERATE_TYPE=""
PACKAGE=""
USERNAME=""
USERGROUP=""
# 宿主机打docker包场景，配置docker文件系统根路径
DOCKER_ROOT=""

while true; do
    case "$1" in
    --copy | -c)
        OPERATE_TYPE="copy"
        shift
        ;;
    --mkdir | -m)
        OPERATE_TYPE="mkdir"
        shift
        ;;
    --makedir | -d)
        OPERATE_TYPE="makedir"
        shift
        ;;
    --chmoddir | -o)
        OPERATE_TYPE="chmoddir"
        shift
        ;;
    --restoremod | -e)
        OPERATE_TYPE="restoremod"
        shift
        ;;
    --remove | -r)
        OPERATE_TYPE="remove"
        shift
        ;;
    --add-env-rc)
        OPERATE_TYPE="add-env-rc"
        shift
        ;;
    --del-env-rc)
        OPERATE_TYPE="del-env-rc"
        shift
        ;;
    --pkg-in-dbinfo)
        OPERATE_TYPE="pkg-in-dbinfo"
        shift
        ;;
    --add-pkg-blocks)
        OPERATE_TYPE="add-pkg-blocks"
        shift
        ;;
    --del-pkg-blocks)
        OPERATE_TYPE="del-pkg-blocks"
        shift
        ;;
    --print-blocks)
        OPERATE_TYPE="print-blocks"
        shift
        ;;
    --print=*)
        OPERATE_TYPE="print"
        PRINT_OPERATE_TYPE="$(echo "$1" | cut -d"=" -f2)"
        shift
        ;;
    --print| -p)
        OPERATE_TYPE="print"
        PRINT_OPERATE_TYPE="copy mkdir"
        shift
        ;;
    --username=*)
        USERNAME=$(echo "$1" | cut -d"=" -f2)
        shift
        ;;
    --usergroup=*)
        USERGROUP=$(echo "$1" | cut -d"=" -f2)
        shift
        ;;
    --install_for_all)
        INSTALL_FOR_ALL="y"
        shift
        ;;
    --setenv)
        SETENV="y"
        shift
        ;;
    --set-cann-uninstall)
        SET_CANN_UNINSTALL="y"
        shift
        ;;
    --package=*)
        PACKAGE=$(echo "$1" | cut -d"=" -f2)
        shift
        ;;
    --docker-root=*)
        DOCKER_ROOT=$(echo "$1" | cut -d"=" -f2)
        shift
        ;;
    -h | --help)
        help_info
        ;;
    -*)
        echo Unrecognized input options : "$1"
        help_info
        ;;
    *)
        break
        ;;
    esac
done

inner_commands "$@"
env_rc_commands "$@"
dbinfo_commands "$@"

if [ $# -lt 3 ]; then
    log "ERROR" "It's too few input params: $*"
    exit 1
fi

formal_commands_with_lock "$@"

exit 0
