#!/bin/bash

installInfo="/etc/ascend_install.info"
ASCEND_SECLOG="/var/log/ascend_seclog"
bootLogFile="${ASCEND_SECLOG}/ascend_run_servers.log"
upgradeLogFile="${ASCEND_SECLOG}/ascend_install.log"
OPERATION_LOGPATH="${ASCEND_SECLOG}/operation.log"
LOG_OPERATION_UPGRADE="Upgrade"
LOG_LEVEL_MINOR="MINOR"
LOG_RESULT_FAILED="failed"
LOG_RESULT_SUCCESS="success"

createLogFile() {
    if [ ! -d /var/log ]; then
        mkdir -p /var/log
        chmod 755 /var/log
    fi
    if [ ! -d "${ASCEND_SECLOG}" ]; then
        mkdir -p ${ASCEND_SECLOG}
        chmod 750 ${ASCEND_SECLOG}
    fi
    if [ ! -f $bootLogFile ]; then
        echo "$cur_time create log file" > $bootLogFile
    else
        fisize=$(stat -c %s $bootLogFile)
        if [ $((fisize)) -gt 5242880 ]; then
            rm -f $bootLogFile
            echo "$cur_time create log file" > $bootLogFile
        fi
    fi
    if [ ! -f $upgradeLogFile ]; then
        echo "$cur_time create log file" > $upgradeLogFile
    fi
    chmod 640 $bootLogFile $upgradeLogFile
}

log() {
    cur_date=`date +"%Y-%m-%d %H:%M:%S"`
    echo "[$cur_date] "$1 >> $bootLogFile
}

#mini start to init
system_init()
{
    if [ -e /var/davinci/scripts/minirc_sys_init.sh ]; then
        log "[INFO]system start to init"
        chmod 550 /var/davinci/scripts/minirc_sys_init.sh
        /var/davinci/scripts/minirc_sys_init.sh >> $bootLogFile 2>&1
        log "[INFO]system init finish"
    else
        log "[ERROR]system init fail"
    fi
}

set_nve_level()
{
    if [ -e /var/nve.tmp ]; then
        . /var/nve.tmp
        if [ ${RUN_TYPE} == "mini" ]; then
            log "[INFO]nve level set start, type is \"${RUN_TYPE}\""
            if [ ! -z ${UPGRADE_NVE_LEVEL} ]; then
                log "[INFO]run type \"${RUN_TYPE}\", set nve level \"${UPGRADE_NVE_LEVEL}\""
                /var/davinci/driver/upgrade-tool --device_index -1 --component nve --level ${UPGRADE_NVE_LEVEL}
                if [ $? -ne 0 ]; then
                    log "[ERROR]nve level set failed"
                fi
            else
                log "[INFO]run type \"${RUN_TYPE}\", set nve level default"
                /var/davinci/driver/upgrade-tool --device_index -1 --component nve --level full
                if [ $? -ne 0 ]; then
                    log "[ERROR]nve level set failed"
                fi
            fi
            rm -rf /var/nve.tmp
            log "[INFO]nve level set finish"
        fi
    fi
}

#start to upgrade firmware
upgrade_firmware()
{
    set_nve_level
    if [ ! -e /var/mini_upgraded ]; then
        log "[INFO]firmware upgrade start"
        /var/davinci/scripts/minirc_firmware_upgrade.sh >> $upgradeLogFile 2>&1
        if [ $? -ne 0 ]; then
            log "[ERROR]firmware upgrade failed"
            logOperation "${start_time}" "${PKG_FILE}-firmware" "${LOG_RESULT_FAILED}"
            return
        fi
        log "[INFO]firmware upgrade finish"
        logOperation "${start_time}" "${PKG_FILE}-firmware" "${LOG_RESULT_SUCCESS}"
        trap : INT QUIT TERM
        reboot
    fi
    if [ -e /sbin/ifup ]; then
        ifup eth0
    else
        ifconfig eth0 up
    fi
}

get_boot_partition() {
    boot_cmd_path=/proc/cmdline
    if [ -e ${boot_cmd_path} ]; then
        boot_area=`cat /proc/cmdline | awk '{for(i=1;i<=NF;i++){split($i,cmd_str,"="); if(cmd_str[1]=="slaveboot"){print cmd_str[2]}}}'`
        if [ $boot_area -eq 1 ]; then
            partition_flag="backup"
        fi
    fi
}

sdk_install()
{
    #start to install new version
    if [ -d ${INSTALL_CACHE_DIR} ]; then
        log "[INFO]driver upgrade start"

        if [ -d ${INSTALL_CACHE_DIR}/mini_developerkit ]; then
            log "detect version rollback from c7x to c3x"
            partition_flag="main"
            get_boot_partition
            if [ "$partition_flag" = "backup" ];then
                log "[WARN]current boot from backup, driver upgrade skiped."
            else
                if [ -e /var/davinci/firmware/upgrade_failed ]; then
                    log "[WARN]c7x crollback to c3x failed, skip booting to wait boot from backup."
                    exit 0
                fi
                chmod 550 ${INSTALL_CACHE_DIR}/mini_developerkit/scripts/*
                ${INSTALL_CACHE_DIR}/mini_developerkit/scripts/minirc_install.sh $INSTALL_CACHE_DIR >> $upgradeLogFile 2>&1
            fi
        else
            chmod 550 ${INSTALL_CACHE_DIR}/driver/scripts/*
            ${INSTALL_CACHE_DIR}/driver/scripts/minirc_install.sh $INSTALL_CACHE_DIR >> $upgradeLogFile 2>&1
        fi

        if [ $? -ne 0 ]; then
            log "[ERROR]driver upgrade failed"
            logOperation "${start_time}" "${PKG_FILE}-driver" "${LOG_RESULT_FAILED}"
        else
            log "[INFO]driver upgrade finish"
            logOperation "${start_time}" "${PKG_FILE}-driver" "${LOG_RESULT_SUCCESS}"
        fi

        #remove rar file
        rm -rf $INSTALL_CACHE_DIR >/dev/null
    fi
}

logOperation() {
    local operation=${LOG_OPERATION_UPGRADE}
    local start_time="$1"
    local runfilename="$2"
    local result="$3"
    local level=${LOG_LEVEL_MINOR}

    if [ ! -f "${OPERATION_LOGPATH}" ]; then
        touch ${OPERATION_LOGPATH}
        chmod 640 ${OPERATION_LOGPATH}
    fi
    echo "${operation} ${level} root ${start_time} 127.0.0.1 ${runfilename} ${result}" >> ${OPERATION_LOGPATH}
}

updateInstallInfo() {
    if [ -f $installInfo ]; then
        . $installInfo
        if [ -z $Driver_Install_Path_Param ]; then
            echo "Driver_Install_Path_Param=/var/davinci" >> $installInfo
        fi
    else
        echo "Driver_Install_Path_Param=/var/davinci" > $installInfo
    fi
    chmod 644 $installInfo
}
# ---------------- start ------------
PKG_FILE=$1
start_time=$(date +"%Y-%m-%d %H:%M:%S")
INSTALL_CACHE_DIR="/var/mini/install_cache"
createLogFile
trap "logOperation \"${start_time}\" \"${PKG_FILE}\" \"${LOG_RESULT_FAILED}\";exit 1" INT QUIT TERM
log "[INFO]mini boot start"
sdk_install
system_init
upgrade_firmware
updateInstallInfo
echo "led3:1led4:1" > /var/davinci/driver/led
log "[INFO]mini boot finish"
exit 0
