#!/bin/bash
ASCEND_SECLOG="/var/log/ascend_seclog"
logFile="${ASCEND_SECLOG}/ascend_install.log"
include_path="/usr/local/Ascend/include"

log() {
    cur_date=`date +"%Y-%m-%d %H:%M:%S"`
    echo "[driver] [$cur_date] "$1 >> $logFile
}

copy_files()
{
        local args_num=$#
        local file=$*
        local num=$(expr $args_num - 1)
        file=($file)
        local target=${file[$num]}
        if echo $target | grep -Eq "*/$"; then
         mkdir -p $target
        fi
        for ((i=0;i<"$num";i=$i+1))
        do
            cp ${INSTALL_CACHE_DIR}/driver/${file[$i]} $target 2>> $logFile
            if [ $? -ne 0 ]; then
                log "[WARNING]copy ${file[$i]} to $target fail"
            else
                log "[INFO]copy ${file[$i]} to $target succ"
            fi
        done

}


backup_old_version()
{
    if [ -e /var/davinci/driver/version.info ]; then
        mkdir -p /var/old_version_backup
        rm -rf /var/old_version_backup/*
        cp -r /var/davinci/driver /var/old_version_backup/
        cp -r /var/davinci/scripts /var/old_version_backup/
        if [ -d /usr/local/Ascend/firmware ]; then
            cp -r /usr/local/Ascend/firmware /var/old_version_backup/
        fi
        cp -r $ASCEND_SECLOG /var/old_version_backup/
    fi
}

delete_files()
{
    local args_num=$#
    local file=$*
    local num=$(expr $args_num - 1)
    file=($file)
    local target=${file[$num]}

    for ((i=0;i<"$num";i=$i+1))
    do
        if [ -e ${target}/${file[$i]} ]; then
            rm ${target}/${file[$i]} 2>> $logFile
            if [ $? -ne 0 ]; then
                log "[WARNING]delete ${target}/${file[$i]} fail"
            else
                log "[INFO]delete ${target}/${file[$i]} succ"
            fi
        fi
    done
}

clear_c3x_version()
{
    if [ -f /etc/sys_version.conf ]; then
        local sys_version=`cat /etc/sys_version.conf`
        sys_version=${sys_version:0:3}
        if [ $sys_version == "1.3" ];then
            log "[INFO]Detect upgrade from c3x, start clear old version file."
        else
            log "[INFO]Detect upgrade from not c3x version, skip clear."
            return
        fi
    else
        log "[INFO]Non-upgrade detected, skip clear."
        return
    fi

    if [ -f /usr/bin/ide_cmd.sh ];then
        chattr -i /usr/bin/ide_cmd.sh
    fi

    local files_lib64="libhiai_python_device.so"
    local files_var="processmonitor matrixdaemon tee.bin aicpufw_monitor hdcd tee-supplicant
            1c3b570d-95ec-4352-bbfe-6d4e58bcd62b.ta nve.bin IDE-daemon-device IDE-daemon-host sercure_tool
            tsch_fw.bin tsch_fw.asm tsch_fw.map dmp_daemon libaicpu.a libcce_aicpudev_alg.a rts_check libc_sec.a
            dt.img sklogd slogd libmedia_server 3559_upgrade_tool minirc_led minirc_pcb usb_ether.sh"
    local files_usr_lib64="libdrvaicpu.so libdrvdevdrv.so libdrvdmp.so libdrvdsmi.so
            libdrvhdc.so libdrvhdc_host.so libdrvlog.so libdrvprof.so libdrvtcp.so libdrvupgrade.so libdrvvos.so
            libDvpp_api.so libDvpp_jpeg_decoder.so libDvpp_jpeg_encoder.so libDvpp_png_decoder.so libDvpp_vpc.so
            libOMX_hisi_video_decoder.so libOMX_hisi_video_encoder.so
            libaicpu_engine.so libcce.so libcce_aicore.so libcce_aicpudev.so libcce_aicpudev_alg.a
            libc_sec.so libssl.so.1.1 libcrypto.so.1.1 libruntime.so libdevmm.so
            libmodel_encrypt_tool.so libteec.so.1.0 libfmk_common.so libgflags.so.2.2
            libidedaemon.so libhiai_client.so libhiai_common.so libhiai_server.so
            libmatrixdaemon.so libproc_bbox.so libmmpa.so libome.so libomg.so
            libopencv_world.so libopencv_world.so.3.4 libopencv_world.so.4.1
            libprofilerclient.so libprofilerserver.so libprotobuf-c.so.1 libprotobuf.so.15
            librts_check.so libslog.so libmedia_mini.so libfusionengine.so libtiling.so libgraph.so libmemory.so
            hiaiengine-py2.7.egg hiaiengine-py3.5.egg libhiai_python_host2.7.so libhiai_python_host3.5.so"
    local files_usr_bin="ide_cmd.sh"
    local files_etc="hdcBasic.cfg sys_version.conf slog.conf"
    local files_home="ide_daemon.cfg ide_daemon_cacert.pem ide_daemon_client_cert.pem ide_daemon_client_key.pem
            ide_daemon_server_cert.pem ide_daemon_server_key.pem"
    local ide_home_path=`eval echo "~${username}"`/ide_daemon

    delete_files $files_lib64         /lib64
    delete_files $files_var           /var
    delete_files $files_usr_lib64     /usr/lib64
    delete_files $files_usr_bin       /usr/bin
    delete_files $files_etc           /etc
    delete_files $files_home          $ide_home_path
}

subdir_create() {
    path=$1
    if [ ! -d $path ];then
        mkdir -p $path
    fi

    chmod 750 $path
    chown -Rf $username:$usergroup $path >> /dev/null 2>&1
    return 0
}

changeMode(){
    chown -f ${username}:${usergroup} $user_home_path ${user_home_path}/.bashrc ${user_home_path}/.bash_logout ${user_home_path}/.profile
    find $bbox_path -name history.log | xargs chmod u+w >> /dev/null 2>&1
    chown -Rf root:root /opt/mini/
    chmod -f 550 /var/tee.bin /var/nve.bin /var/dt.img
    chown -Rf ${username}:${usergroup} /var/tee.bin /var/nve.bin /var/dt.img
    chmod -f 444 /var/davinci/driver/version.info
    chmod -f 640 /var/davinci/driver/led
    chmod -f 640 /var/log/ascend_seclog/boot_fail_count
    chmod -f 550 /fw
    chmod -f 440 /fw/*
    chmod -f 750 /var/old_version_backup
    chown -RPf ${username}:${usergroup} /var/log/npu/profiling
    chown -RPf ${username}:${usergroup} /var/log/npu/conf/profiling
    chown -RPf ${username}:${usergroup} /var/log/npu/slog/slogd/
    chown -RPf ${username}:${usergroup} /var/log/npu/slog/device-0/
    chown -RPf ${username}:${usergroup} /var/log/npu/slog/host-0
}

set_security_configuration() {
    local set_write_flag="false"
    if [ ! -w "/etc/sudoers" ];then
        chmod u+w /etc/sudoers
        set_write_flag="true"
    fi

    grep -q "HwHiAiUser" /etc/sudoers
    if [ $? -eq 0 ];then
        sed -i '/HwHiAiUser/c\HwHiAiUser ALL=(root) NOPASSWD:/opt/mini/minirc_install_phase1.sh,/var/add_aicpu_tid_to_tasks.sh,/usr/local/Ascend/driver/tools/minirc_container_prepare.sh,/bin/date -s *,/bin/sed -i * /etc/network/interfaces,/usr/bin/pkill -2 perf,/usr/bin/perf script -F comm\\,pid\\,tid\\,cpu\\,time\\,period\\,event\\,ip\\,sym\\,dso\\,symoff -i /var/log/npu/profiling/[0-9]/ai_ctrl_cpu.data.[0-9].[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9] --show-kernel-path -f,/usr/bin/perf record -o /var/log/npu/profiling/[0-9]/ai_ctrl_cpu.data.[0-9] -F [0-9][0-9][0-9][0-9]? -N -B -T -g -e?{r8\\,r11}? -a --switch-output=10s,/usr/bin/perf stat -o /var/log/npu/profiling/[0-9]/llc.data.[0-9] -a -e?hisi_l3c0_1/dsid0/\\,hisi_l3c0_1/dsid1/\\,hisi_l3c0_1/dsid2/\\,hisi_l3c0_1/dsid3/\\,hisi_l3c0_1/dsid4/\\,hisi_l3c0_1/dsid5/\\,hisi_l3c0_1/dsid6/\\,hisi_l3c0_1/dsid7/?  -I [0-9] ,/usr/bin/perf stat -o /var/log/npu/profiling/[0-9]/llc.data.[0-9] -a -e?hisi_l3c0_1/dsid0/\\,hisi_l3c0_1/dsid1/\\,hisi_l3c0_1/dsid2/\\,hisi_l3c0_1/dsid3/\\,hisi_l3c0_1/dsid4/\\,hisi_l3c0_1/dsid5/\\,hisi_l3c0_1/dsid6/\\,hisi_l3c0_1/dsid7/?  -I [0-9][0-9] ,/usr/bin/perf stat -o /var/log/npu/profiling/[0-9]/llc.data.[0-9] -a -e?hisi_l3c0_1/dsid0/\\,hisi_l3c0_1/dsid1/\\,hisi_l3c0_1/dsid2/\\,hisi_l3c0_1/dsid3/\\,hisi_l3c0_1/dsid4/\\,hisi_l3c0_1/dsid5/\\,hisi_l3c0_1/dsid6/\\,hisi_l3c0_1/dsid7/?  -I [0-9][0-9][0-9],/usr/bin/perf stat -o /var/log/npu/profiling/[0-9]/llc.data.[0-9] -a -e?hisi_l3c0_1/dsid0/\\,hisi_l3c0_1/dsid1/\\,hisi_l3c0_1/dsid2/\\,hisi_l3c0_1/dsid3/\\,hisi_l3c0_1/dsid4/\\,hisi_l3c0_1/dsid5/\\,hisi_l3c0_1/dsid6/\\,hisi_l3c0_1/dsid7/?  -I [0-9][0-9][0-9][0-9] ,/usr/bin/perf stat -o /var/log/npu/profiling/[0-9]/llc.data.[0-9]  -a -e?hisi_l3c0_1/read_allocate/\\,hisi_l3c0_1/read_hit/\\,hisi_l3c0_1/read_noallocate/\\,hisi_l3c0_1/write_allocate/\\,hisi_l3c0_1/write_hit/\\,hisi_l3c0_1/write_noallocate/? -I [0-9] ,/usr/bin/perf stat -o /var/log/npu/profiling/[0-9]/llc.data.[0-9]   -a -e?hisi_l3c0_1/read_allocate/\\,hisi_l3c0_1/read_hit/\\,hisi_l3c0_1/read_noallocate/\\,hisi_l3c0_1/write_allocate/\\,hisi_l3c0_1/write_hit/\\,hisi_l3c0_1/write_noallocate/? -I [0-9][0-9],/usr/bin/perf stat -o /var/log/npu/profiling/[0-9]/llc.data.[0-9]   -a -e?hisi_l3c0_1/read_allocate/\\,hisi_l3c0_1/read_hit/\\,hisi_l3c0_1/read_noallocate/\\,hisi_l3c0_1/write_allocate/\\,hisi_l3c0_1/write_hit/\\,hisi_l3c0_1/write_noallocate/? -I [0-9][0-9][0-9],/usr/bin/perf stat -o /var/log/npu/profiling/[0-9]/llc.data.[0-9]   -a -e?hisi_l3c0_1/read_allocate/\\,hisi_l3c0_1/read_hit/\\,hisi_l3c0_1/read_noallocate/\\,hisi_l3c0_1/write_allocate/\\,hisi_l3c0_1/write_hit/\\,hisi_l3c0_1/write_noallocate/? -I [0-9][0-9][0-9][0-9]' /etc/sudoers
    else
        echo "HwHiAiUser ALL=(root) NOPASSWD:/opt/mini/minirc_install_phase1.sh,/var/add_aicpu_tid_to_tasks.sh,/bin/date -s *,/bin/sed -i * /etc/network/interfaces,/usr/bin/pkill -2 perf,/usr/bin/perf script -F comm\\,pid\\,tid\\,cpu\\,time\\,period\\,event\\,ip\\,sym\\,dso\\,symoff -i /var/log/npu/profiling/[0-9]/ai_ctrl_cpu.data.[0-9].[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9] --show-kernel-path -f,/usr/bin/perf record -o /var/log/npu/profiling/[0-9]/ai_ctrl_cpu.data.[0-9] -F [0-9][0-9][0-9][0-9]? -N -B -T -g -e?{r8\\,r11}? -a --switch-output=10s,/usr/bin/perf stat -o /var/log/npu/profiling/[0-9]/llc.data.[0-9] -a -e?hisi_l3c0_1/dsid0/\\,hisi_l3c0_1/dsid1/\\,hisi_l3c0_1/dsid2/\\,hisi_l3c0_1/dsid3/\\,hisi_l3c0_1/dsid4/\\,hisi_l3c0_1/dsid5/\\,hisi_l3c0_1/dsid6/\\,hisi_l3c0_1/dsid7/?  -I [0-9] ,/usr/bin/perf stat -o /var/log/npu/profiling/[0-9]/llc.data.[0-9] -a -e?hisi_l3c0_1/dsid0/\\,hisi_l3c0_1/dsid1/\\,hisi_l3c0_1/dsid2/\\,hisi_l3c0_1/dsid3/\\,hisi_l3c0_1/dsid4/\\,hisi_l3c0_1/dsid5/\\,hisi_l3c0_1/dsid6/\\,hisi_l3c0_1/dsid7/?  -I [0-9][0-9] ,/usr/bin/perf stat -o /var/log/npu/profiling/[0-9]/llc.data.[0-9] -a -e?hisi_l3c0_1/dsid0/\\,hisi_l3c0_1/dsid1/\\,hisi_l3c0_1/dsid2/\\,hisi_l3c0_1/dsid3/\\,hisi_l3c0_1/dsid4/\\,hisi_l3c0_1/dsid5/\\,hisi_l3c0_1/dsid6/\\,hisi_l3c0_1/dsid7/?  -I [0-9][0-9][0-9],/usr/bin/perf stat -o /var/log/npu/profiling/[0-9]/llc.data.[0-9] -a -e?hisi_l3c0_1/dsid0/\\,hisi_l3c0_1/dsid1/\\,hisi_l3c0_1/dsid2/\\,hisi_l3c0_1/dsid3/\\,hisi_l3c0_1/dsid4/\\,hisi_l3c0_1/dsid5/\\,hisi_l3c0_1/dsid6/\\,hisi_l3c0_1/dsid7/?  -I [0-9][0-9][0-9][0-9] ,/usr/bin/perf stat -o /var/log/npu/profiling/[0-9]/llc.data.[0-9]  -a -e?hisi_l3c0_1/read_allocate/\\,hisi_l3c0_1/read_hit/\\,hisi_l3c0_1/read_noallocate/\\,hisi_l3c0_1/write_allocate/\\,hisi_l3c0_1/write_hit/\\,hisi_l3c0_1/write_noallocate/? -I [0-9] ,/usr/bin/perf stat -o /var/log/npu/profiling/[0-9]/llc.data.[0-9]   -a -e?hisi_l3c0_1/read_allocate/\\,hisi_l3c0_1/read_hit/\\,hisi_l3c0_1/read_noallocate/\\,hisi_l3c0_1/write_allocate/\\,hisi_l3c0_1/write_hit/\\,hisi_l3c0_1/write_noallocate/? -I [0-9][0-9],/usr/bin/perf stat -o /var/log/npu/profiling/[0-9]/llc.data.[0-9]   -a -e?hisi_l3c0_1/read_allocate/\\,hisi_l3c0_1/read_hit/\\,hisi_l3c0_1/read_noallocate/\\,hisi_l3c0_1/write_allocate/\\,hisi_l3c0_1/write_hit/\\,hisi_l3c0_1/write_noallocate/? -I [0-9][0-9][0-9],/usr/bin/perf stat -o /var/log/npu/profiling/[0-9]/llc.data.[0-9]   -a -e?hisi_l3c0_1/read_allocate/\\,hisi_l3c0_1/read_hit/\\,hisi_l3c0_1/read_noallocate/\\,hisi_l3c0_1/write_allocate/\\,hisi_l3c0_1/write_hit/\\,hisi_l3c0_1/write_noallocate/? -I [0-9][0-9][0-9][0-9]" >> /etc/sudoers
    fi

    if [ $set_write_flag = "true" ];then
        chmod u-w /etc/sudoers
    fi

    unset set_write_flag
    return 0
}

mkIdePath() {
    if [ ! -d $ide_daemon_path ];then
        mkdir -p $ide_daemon_path
    fi

    if [ ! -d $coredump_path ];then
        mkdir -p $coredump_path
    fi

    chmod 700 $ide_daemon_path
    chmod 700 $coredump_path
    chown -RPf $username:$usergroup $ide_daemon_path
    chown -RPf $username:$usergroup $coredump_path
}

getDevname() {
    local tmp_name=`cat /proc/cmdline | tr -s ' ' '\n' | grep "root=" | awk -F '=' '{print $2}' | awk -F 'p' '{print $1}'`
    if [ -e "${tmp_name}" ]; then
        devname=${tmp_name}
    fi
}

INSTALL_CACHE_DIR=$1

username=HwHiAiUser
usergroup=HwHiAiUser
conf_path=/var/log/npu/conf/
log_path=/var/log/npu/slog/
bbox_path=/var/log/npu/hisi_logs
user_home_path=`eval echo "~${username}"`
ide_daemon_path=`eval echo "~${username}"`/ide_daemon
coredump_path=/var/log/npu/coredump
devname="/dev/mmcblk1"
# ---------------- copy start ------------
backup_old_version
clear_c3x_version
[ -d /var/davinci ] && rm -rf /var/davinci
[ -f /usr/bin/perf ] && rm /usr/bin/perf

log "[INFO]create log dir and conf dir"
cd ${INSTALL_CACHE_DIR}/driver
scripts/install_common_parser.sh --makedir --username=$username --usergroup=$usergroup all . scripts/filelist.csv 2>>$logFile
if [ $? -ne 0 ];then
    log "[ERROR]driver mkdir failed"
    exit 1
fi
log "[INFO]driver mkdir successfully"

getDevname
grep -qs $log_path /proc/mounts >> /dev/null 2>&1
ret=$?
if [ -e "${devname}p2" ] && [ $ret -ne 0 ]; then
    mkdir /var/oldlog
    rsync -a --include '*/' --exclude '*'  $log_path /var/oldlog/ >& /dev/null
    mount "${devname}p2" $log_path
    rsync -a --include '*/' --exclude '*'  /var/oldlog/ $log_path >& /dev/null
    rm -rf /var/oldlog
fi

grep -qs '/home' /proc/mounts >> /dev/null 2>&1
ret=$?
if [ -e "${devname}p3" ] && [ $ret -ne 0 ]; then
    mount "${devname}p3" /home
fi

log "[INFO]copy files to target form cache"
scripts/install_common_parser.sh --copy --username=$username --usergroup=$usergroup all . scripts/filelist.csv 2>>$logFile
if [ $? -ne 0 ];then
    log "[ERROR]driver copy failed"
    exit 1
fi
scripts/install_common_parser.sh --chmoddir --username=$username --usergroup=$usergroup all . scripts/filelist.csv 2>>$logFile
if [ $? -ne 0 ];then
    log "[ERROR]driver chmoddir  failed"
    exit 1
fi
mkIdePath
cp ${INSTALL_CACHE_DIR}/driver/version.info /var/davinci/driver
cp ${INSTALL_CACHE_DIR}/driver/scripts/minirc_install_config.conf /var/davinci/scripts
mv /usr/bin/perf_rc /usr/bin/perf
log "[INFO]driver copy successfully"

su - $username -c "cd $include_path >> /dev/null 2>&1"
if [ $? -ne 0 ]; then
    log "[ERROR]The $username do not have the permission to access $include_path, please reset the directory to a right permission."
    exit 1
fi

# ---------------- set user conf start ------------
if [ -e ${INSTALL_CACHE_DIR}/driver/scripts/userfilelist.csv ];then
    log "[INFO]create user dir and copy files to target form cache"
    scripts/install_common_parser.sh --mkdir --username=$username --usergroup=$usergroup all . scripts/userfilelist.csv 2>>$logFile
    if [ $? -ne 0 ];then
        log "[WARN]user file list mkdir failed or file is null"
    fi

    scripts/install_common_parser.sh --copy --username=$username --usergroup=$usergroup all . scripts/userfilelist.csv 2>>$logFile
    if [ $? -ne 0 ];then
        log "[WARN]user file list copy failed or file is null"
    fi

    scripts/install_common_parser.sh --chmoddir --username=$username --usergroup=$usergroup all . scripts/userfilelist.csv 2>>$logFile
    if [ $? -ne 0 ];then
        log "[WARN]user file list chmod failed or file is null"
    fi
    log "[INFO]set user conf finished"
fi

# ----------------- set npu-smi -------------------
if [ -f ${INSTALL_CACHE_DIR}/driver/scripts/minirc_install_customized.sh ]; then
    chmod +x ${INSTALL_CACHE_DIR}/driver/scripts/minirc_install_customized.sh
    ${INSTALL_CACHE_DIR}/driver/scripts/minirc_install_customized.sh ${INSTALL_CACHE_DIR}
    if [ $? -ne 0 ]; then
        log "install customized failed"
    fi
fi

set_security_configuration
changeMode
cd - >>/dev/null 2>&1
exit 0
