#!/bin/bash
drv_filelist="/var/davinci/scripts/filelist.csv"
usr_filelist="/var/davinci/scripts/userfilelist.csv"
copy_script="/var/davinci/scripts/install_common_parser.sh"
backup_conf="/var/davinci/scripts/backup_file_list.conf"
backup_dir="/data/fs_backup/"
log_file="/var/log/ascend_seclog/ascend_run_servers.log"
username=HwHiAiUser
usergroup=HwHiAiUser

log() {
    cur_date=`date +"%Y-%m-%d %H:%M:%S"`
    echo "[driver] [$cur_date] "$1 >> $log_file
}

do_mkdir() {
    local operation="mkdir"
    local csv_file=$1
    tmp_csv="${operation}.csv"
    awk -F ',' '$2==op' op=${operation} ${csv_file} | \
        awk 'BEGIN{FS=OFS=","}{$4=dst$4}1' dst=${backup_dir} > ${tmp_csv}
    ${copy_script} --${operation} --username=${username} --usergroup=${usergroup} all . ${tmp_csv}
    if [ $? -ne 0 ]; then
        log "[ERROR]copy failed."
        rm ${tmp_csv}
        return 1
    fi
    ${copy_script} --chmoddir --username=${username} --usergroup=${usergroup} all . ${tmp_csv}
    if [ $? -ne 0 ]; then
        log "[ERROR]chmod failed."
        rm ${tmp_csv}
        return 1
    fi
    rm ${tmp_csv}
}

do_copy() {
    local operation="copy"
    local csv_file=$1
    tmp_csv="${operation}.csv"
    awk -F ',' '$2==op' op=${operation} ${csv_file} | \
        awk 'BEGIN{FS=OFS=","}{$3=$4}{$4=dst$4}1' dst=${backup_dir} > ${tmp_csv}

    # perf_rc images no need to copy to backup part
    if [ "${csv_file}" == "${drv_filelist}" ]; then
        sed -i '\?/usr/bin/perf_rc?d' ${tmp_csv}
        sed -i '\?/var/davinci/firmware?d' ${tmp_csv}
    fi
    ${copy_script} --${operation} --username=${username} --usergroup=${usergroup} all . ${tmp_csv}
    if [ $? -ne 0 ]; then
        log "[ERROR]copy failed."
        rm ${tmp_csv}
        return 1
    fi
    ${copy_script} --chmoddir --username=${username} --usergroup=${usergroup} all . ${tmp_csv}
    if [ $? -ne 0 ]; then
        log "[ERROR]chmod failed."
        rm ${tmp_csv}
        return 1
    fi
    rm ${tmp_csv}
}

copy_backup_file() {
    while read line
    do
        if [ -e "${line}" ]; then
            if [ -d "${line}" ]; then
                # When using rsync to synchronize directories, add / to the end of directories.
                rsync -ar --exclude "*.swp" ${line}/ ${backup_dir}${line}/ || log "[ERROR]${line} copy failed."
            else
                rsync -a ${line} ${backup_dir}${line} || log "[ERROR]${line} copy failed."
            fi
        else
            log "[WARNING]${line} no exist."
        fi
    done < ${backup_conf}
}

copy_backup_file

do_mkdir "${drv_filelist}" || exit 1
do_mkdir "${usr_filelist}" || exit 1
do_copy "${drv_filelist}" || exit 1
do_copy "${usr_filelist}" || exit 1
