#!/bin/bash
watch_conf=$1
watch_log="/var/log/ascend_seclog/ascend_run_servers.log"
watch_list=()
backup_dir="/data/fs_backup/"

function log() {
	cur_date=`date +"%Y-%m-%d %H:%M:%S"`
	echo "[driver] [$cur_date] "$1 >> $watch_log
}

function WatchFileCehck(){
	if [ ! -f "$watch_conf" ]; then
		log "[ERROR]$watch_conf:No such file or directory."
		return
	fi

	while read line
	do
		if [ ! -f "$line" ] && [ ! -d "$line" ]; then
			log "[INFO]The file or directory($line) does not exist and will not be monitored."
			continue
		fi

		watch_list+=($line)
	done < $watch_conf
}

if [ $# -lt 1 ];then
	log "[ERROR]Invalid Parament."
	exit 1
fi

type inotifywait
if [ $? != 0 ];then
	log "[ERROR]Command inotifywait not installed."
	exit 1
fi

while [ 1 ]
do
	WatchFileCehck
	if [ ${#watch_list[@]} == 0 ];then
		log "[INFO]$watch_conf is empty."
		sleep 1
		continue
	fi
	inotifywait -rq --exclude "(^\..*|.*\.swp.*$)" -e create,delete,move,close_write,delete_self --timefmt '%y/%m/%d-%H:%M' --format '%T %w %f %e' ${watch_list[@]} | \
	while read TIME DIR FILE EVENT
	do
		log "$TIME $DIR $FILE $EVENT"
		TargetDir=`realpath $DIR`
		sync_list=`echo ${watch_list[@]} | tr -s ' ' '\n' | grep $TargetDir`
		for file in $sync_list
		do
			if [ -d "$file" ];then
                                # When using rsync to synchronize directories, add / to the end of directories.
				rsync -ar --exclude "*.swp" ${file}/ ${backup_dir}${file}/
			else
				rsync -a ${file} ${backup_dir}${file}
			fi
			if [ $? -ne 0 ]; then
				log "[ERROR]Sync failed, ${file} to /data/fs_backup${file} failed."
			else
				log "[INFO]Sync success, ${file} to /data/fs_backup${file} success."
			fi
		done
	done
	unset watch_list
done
