#!/bin/bash
ASCEND_SECLOG="/var/log/ascend_seclog"
logFile="${ASCEND_SECLOG}/ascend_install.log"
install_conf_file=/var/davinci/scripts/minirc_install_config.conf
nve_level="full"
run_type=""

log() {
    cur_date=`date +"%Y-%m-%d %H:%M:%S"`
    echo "[firmware] [$cur_date] "$1 >> $logFile
}

set_led_state()
{
    #stats  booting  upgrading finish
    state=$1
    if [ $state == "booting" ]; then
        echo "led3:1led4:0" > /var/davinci/driver/led
    elif [ $state == "upgrading" ]; then
        echo  "led3:2led4:2" > /var/davinci/driver/led
    elif [ $state == "finish" ]; then
        echo "led3:1led4:1" > /var/davinci/driver/led
    fi
}

minirc_upgrade_progress()
{
    fw=$1
    status=$2
    percent=$3

    if [ $status -eq 0 ]; then
        log "[INFO]$fw upgrade succ ----------------- $percent%"
    else
        echo "$(date +%Y-%m-%d,%H:%M:%S)  firmware upgraded failed" > /var/mini_upgraded
        set_led_state "finish"
        log "[ERROR]$fw upgrade result:failed"
        exit 1
    fi
}

#校验tmp文件中的安装类型信息
get_run_type() {
    if [ -e /var/nve.tmp ]; then
        . /var/nve.tmp
        run_type=$RUN_TYPE
        rm -rf /var/nve.tmp
    fi
}

#获取nve功耗档位
get_nve_level() {
    get_run_type
    if [ ${run_type} == "pc" ]; then
        log "[INFO]nve level get start, type is \"${run_type}\""
        #取出文件配置
        if [ -e ${install_conf_file} ]; then
            . ${install_conf_file}
            nve_level=${nve_rate_level}
            log "[INFO]nve level \"${nve_rate_level}\" get sucess"
        else
            log "[WARNING] get nve level failed, file minirc_install_config.conf doesn't exist."
        fi

        if [ ${nve_level} == "default" ]; then
            nve_level="full"
        fi

        if [ ${nve_level} != "full" ] && [ ${nve_level} != "high" ] && [ ${nve_level} != "middle" ] && [ ${nve_level} != "low" ]; then
            log "[WARNING] config error, config is \"${nve_level}\"."
            nve_level="full"
        fi
        log "[INFO]nve level get finish"
    else
        log "[WARNING] run_type error, run_type is \"${run_type}\", please check nve.tmp."
    fi
}

firmware_upgrade()
{
    sleep 1s
    cd /var/davinci/firmware

    log "[INFO]show version before upgrade"
    /var/davinci/driver/upgrade-tool --device_index -1 --component -1 --version

    get_nve_level
    /var/davinci/driver/upgrade-tool --device_index -1 --component nve --level $nve_level
    /var/davinci/driver/upgrade-tool --device_index -1 --component nve --path ./nve.bin
    minirc_upgrade_progress "nve" $? 30

    /var/davinci/driver/upgrade-tool --device_index -1 --component xloader --path ./xloader.bin
    minirc_upgrade_progress "xloader" $? 60

    /var/davinci/driver/upgrade-tool --device_index -1 --component uefi --path ./HI1910_FPGA_DDR.fd
    minirc_upgrade_progress "uefi" $? 100

    log "[INFO]show version after upgrade"
    /var/davinci/driver/upgrade-tool --device_index -1 --component -1 --version

    #mark firmware upgraded
    echo "$(date +%Y-%m-%d,%H:%M:%S)  firmware upgraded" > /var/mini_upgraded
    return

    #usb scene not support temporarily
    if [ ! -e /var/not_upgrade_fs ]; then
        #the follow componentes are only used in u_disk mode, you can add file(/var/not_upgrade_fs) to stop upgrading these componentes
        /var/davinci/driver/upgrade-tool --device_index -1 --component m3fw --path ./lpm3.img
        minirc_upgrade_progress "m3fw" $? 30

        /var/davinci/driver/upgrade-tool --device_index -1 --component kernel --path ./Image
        minirc_upgrade_progress "kernel" $? 70

        /var/davinci/driver/upgrade-tool --device_index -1 --component tee --path ./tee.bin
        minirc_upgrade_progress "tee" $? 75

        /var/davinci/driver/upgrade-tool --device_index -1 --component rootfs --path ./filesystem-le.cpio.gz
        minirc_upgrade_progress "rootfs" $? 95

        /var/davinci/driver/upgrade-tool --device_index -1 --component dtb --path ./dt.img
        minirc_upgrade_progress "dtb" $? 100
    else
        log "[INFO]user cancel upgrading the componentes of u_disk"
    fi
}

log "[INFO]firmware start to upgrade  ----------------- 0%"

set_led_state "upgrading"
firmware_upgrade

#reboot after upgrade
log "[INFO]firmware upgrade finish, start to reboot"
if [ -d /var/davinci/firmware ]; then
    rm -rf /var/davinci/firmware
fi
set_led_state "finish"
exit 0
