#!/bin/bash
ASCEND_SECLOG="/var/log/ascend_seclog"
logFile="${ASCEND_SECLOG}/ascend_install.log"

log() {
    cur_date=`date +"%Y-%m-%d %H:%M:%S"`
    echo "[driver] [$cur_date] "$1 >> $logFile
}

#set environment variable
set_environment_variable()
{
    file_conf=/etc/ld.so.conf.d/mind_so.conf
    if [ -f $file_conf ];then
        rm $file_conf
        ldconfig
    fi
    echo /usr/lib64 >> $file_conf
    ldconfig
}

check_user(){
    ret=`cat /etc/passwd | cut -f1 -d':' | grep -w "$1" -c`
    if [ $ret -le 0 ]; then
        return 1
    else
        return 0
    fi
}

get_boot_partition() {
    boot_cmd_path=/proc/cmdline
    if [ -e ${boot_cmd_path} ]; then
        boot_area=`cat /proc/cmdline | awk '{for(i=1;i<=NF;i++){split($i,cmd_str,"="); if(cmd_str[1]=="slaveboot"){print cmd_str[2]}}}'`
        fs_partition=`cat /proc/cmdline | tr -s ' ' '\n' | grep "root=" | awk -F '=' '{print $2}'`
        if [ $boot_area -eq 1 ]; then
            partition_flag="backup"
        fi
    fi
}

log "[INFO]driver upgrade start"
username=HwHiAiUser
check_user ${username}
if [ $? -ne 0 ];then
    log "[ERROR]user HwHiAiUser does not exists!"
    exit 1
fi

INSTALL_CACHE_DIR=$1

FW_UPGRADE_RESULT_FILE="/var/davinci/firmware/upgrade_failed"
partition_flag="main"
get_boot_partition
if [ "$partition_flag" = "backup" ] && [[ "${fs_partition}" =~ "p1" ]] ;then
    log "[WARN]current boot from backup, driver upgrade skiped."
    exit 0
fi

if [ ! -d ${INSTALL_CACHE_DIR} ];then
    log "[ERROR]dir ${INSTALL_CACHE_DIR} not exited, driver upgrade failed"
    exit 1
else
    if [ -e $FW_UPGRADE_RESULT_FILE ]; then
        log "[WARN]last firmware upgrade failed, driver upgrade skiped."
        exit 0
    fi

    . $INSTALL_CACHE_DIR/driver/version.info
    log "[INFO]driver start to upgrade new version($Version)"

    #copy files to target
    chmod 550 ${INSTALL_CACHE_DIR}/driver/scripts/*.sh
    ${INSTALL_CACHE_DIR}/driver/scripts/minirc_cp_file.sh $INSTALL_CACHE_DIR 2>> $logFile
    if [ $? -ne 0 ];then
        exit 1
    fi
    #set env value
    set_environment_variable

    log "[INFO]driver upgrade finish"
fi

exit 0
