#!/bin/bash

# the definition of return value
#1: lack enough sd space
#2: unpack tar package failed
#3: version verify failed
#4: undefined error

ASCEND_SECLOG="/var/log/ascend_seclog"
logFile="${ASCEND_SECLOG}/ascend_install.log"
OPERATION_LOGPATH="${ASCEND_SECLOG}/operation.log"
LOG_OPERATION_UPGRADE="Upgrade"
LOG_LEVEL_MINOR="MINOR"
LOG_RESULT_FAILED="failed"
nve_level=""

createLogFile() {
    [ "$RUN_TYPE" == "pc" ] && return
    if [ ! -d /var/log ]; then
        mkdir -p /var/log
        chmod 755 /var/log
    fi
    if [ ! -d "${ASCEND_SECLOG}" ]; then
        mkdir -p ${ASCEND_SECLOG}
        chmod 750 ${ASCEND_SECLOG}
    fi
    if [ ! -f $logFile ]; then
        echo "$cur_time create log file" > $logFile
    fi
    chmod 640 $logFile
}

log() {
    echo $1
    cur_date=`date +"%Y-%m-%d %H:%M:%S"`
    [ "$RUN_TYPE" == "mini" ] && echo "[driver] [$cur_date] "$1 >> $logFile
}

errorLog() {
    log "$1"
    logOperation "${start_time}" "${LOG_RESULT_FAILED}"
    rm -rf ${INSTALL_CACHE_DIR}
    if [ $RUN_TYPE == "mini" ]; then
        rm -f $FW_UPGRADE_RESULT_FILE
    fi
    sync
    exit 1
}

# backup logfile
rotateLog() {
    [ "$RUN_TYPE" == "pc" ] && return
    echo "${logFile} {
        su root root
        daily
        size=5M
        rotate 3
        missingok
        create 440 root root
    }" > /etc/logrotate.d/ascend-install
}

# check install dir space isn't enough, driver package need 200M
check_space()
{
    [ "$RUN_TYPE" == "pc" ] && return
    freeSpace=`df -BM / |awk '/[0-9]%/{print $(NF-2)}'`
    log "[INFO]freeSpace:${freeSpace}"
    if [ ${freeSpace%?} -le $occupy_space ]; then
        log "[ERROR]not enough space left in sdcard, cannot install driver package (at least $occupy_space M)"
        exit 1
    fi
}


#extrack new version package,,tar not keep UID&GID
extrack_package()
{
    if [ -d ${INSTALL_CACHE_DIR} ]; then
        rm -rf ${INSTALL_CACHE_DIR}
        log "[INFO]exsit install_cache_dir,delete the original directory"
    fi

    log "[INFO]extrack package, file name:${PKG_FILE}"
    mkdir -p ${INSTALL_CACHE_DIR}
    if [ "$UPGRADE_TARGET" == "C3X" ]; then
        COMPRESS_TOOL="unzip"
        unzip ${PKG_FILE} -d ${INSTALL_CACHE_DIR} >> /dev/null 2>&1
    else
        COMPRESS_TOOL="tar"
        tar  --no-same-owner -zxf ${PKG_FILE} -C ${INSTALL_CACHE_DIR} >>/dev/null 2>&1
    fi

    if [ $? -ne 0 ]; then
        which $COMPRESS_TOOL
        if [ $? -ne 0 ]; then
            log "[ERROR]can't find $COMPRESS_TOOL tool, please install $COMPRESS_TOOL tool first"
        else
            log "[ERROR]maybe the package is damaged, please check it"
        fi
        errorLog "[ERROR]extrack package(${PKG_FILE}) fail, installation failed"
    else
        log "[INFO]extrack package succ"
    fi
}

device_crl_check() {
    [ "$RUN_TYPE" == "pc" ] && return
    [ "$UPGRADE_TARGET" == "C3X" ] && return
    #check CRL of images
    if [ -f $INSTALL_CACHE_DIR/driver/scripts/device_crl_check.sh ]; then
        $INSTALL_CACHE_DIR/driver/scripts/device_crl_check.sh
        if [ $? -ne 0 ];then
            errorLog "[ERROR]driver crl check fail, stop upgrade"
        fi
    elif [ -f /root/ascend_check/ascend_310.crl ]; then
        errorLog "[ERROR]1.generate new images crl fail, stop install"
    fi
}

install_phase1()
{
    if [ "$UPGRADE_TARGET" == "C3X" ]; then
        chmod 550 $INSTALL_CACHE_DIR/mini_developerkit/scripts/*
        # C3X minirc_install_pre.sh just need one parameter.
        $INSTALL_CACHE_DIR/mini_developerkit/scripts/minirc_install_pre.sh $SDCARD_MOUNT_DIR
    else
        chmod 550 $INSTALL_CACHE_DIR/driver/scripts/*
        $INSTALL_CACHE_DIR/driver/scripts/minirc_install_pre.sh $SDCARD_MOUNT_DIR $RUN_TYPE
    fi
    result=$?
    if [ $result -ne 0 ]; then
        errorLog "[ERROR]write files to sdcard failed,please check"
    fi
}

minirc_upgrade_progress()
{
	fw=$1
	status=$2
	percent=$3
	if [ $status -eq 0 ]; then
		result="succ"
	else
		result="fail"
		upgrade_result=$status
	fi
	echo "$fw upgrade $result ----------------- $percent%" >> $INSTALL_CACHE_FILE_DIR/firmware_upgrade_progress.log
	echo "$fw upgrade result:$status"
}

process_nve_level()
{
    if [ -e "/var/davinci/firmware/opsread-tool" ]; then
        nve_level=`/var/davinci/firmware/opsread-tool`
        echo "[INFO]get old version nve level success, will set nve level as $nve_level."
    elif [ -e "/var/davinci/driver/upgrade-tool" ]; then
        nve_level=`/var/davinci/driver/upgrade-tool --device_index -1 --component nve --level` > /dev/null 2>&1
        if [[ $? -eq 0 ]]; then
            nve_level=`echo ${nve_level} | awk '{print $3}'`
            nve_level=${nve_level:8:${#nve_level}-9}
            echo "[INFO]get old version nve level success, will set nve level as $nve_level."
        else
            echo "[INFO]old version do not support getting nve level, will set nve level as full."
            nve_level="full"
        fi
    else
        echo "[INFO]do not find tools, will set nve level as full."
        nve_level="full"
    fi
    if [[ ${nve_level} != "full" ]] && [[ ${nve_level} != "high" ]] && [[ ${nve_level} != "middle" ]] && [[ ${nve_level} != "low" ]]; then
        echo "[ERROR]get wrong nve level(${nve_level}), please check script, level wiil be set as full."
        nve_level="full"
    fi
}

firmware_upgrade()
{
    if [ $RUN_TYPE == "mini" ]; then
        sleep 1s
        if [ ! -d $INSTALL_CACHE_FILE_DIR ]; then
            echo "[ERROR]$INSTALL_CACHE_FILE_DIR not exist."
            return 1
        fi

        upgrade_result=0
        echo "!!!!!! start to upgrade firmware, don't power off or reset the board !!!"
        $INSTALL_CACHE_TOOL_DIR/upgrade-tool --device_index -1 --component m3fw --media --mmc --main --path $INSTALL_CACHE_FILE_DIR/lpm3.img
        minirc_upgrade_progress "m3fw mmc" $? 5
        $INSTALL_CACHE_TOOL_DIR/upgrade-tool --device_index -1 --component kernel --media --mmc --main --path $INSTALL_CACHE_FILE_DIR/Image
        minirc_upgrade_progress "kernel mmc" $? 15
        $INSTALL_CACHE_TOOL_DIR/upgrade-tool --device_index -1 --component tee --media --mmc --main --path $INSTALL_CACHE_FILE_DIR/tee.bin
        minirc_upgrade_progress "tee mmc" $? 30
        $INSTALL_CACHE_TOOL_DIR/upgrade-tool --device_index -1 --component dtb --media --mmc --main --path $INSTALL_CACHE_FILE_DIR/dt.img
        minirc_upgrade_progress "dtb mmc" $? 50
        process_nve_level
        $INSTALL_CACHE_TOOL_DIR/upgrade-tool --device_index -1 --component nve --path $INSTALL_CACHE_FILE_DIR/nve.bin
        minirc_upgrade_progress "nve flash" $? 75
        $INSTALL_CACHE_TOOL_DIR/upgrade-tool --device_index -1 --component xloader --path $INSTALL_CACHE_FILE_DIR/xloader.bin
        minirc_upgrade_progress "xloader flash" $? 85
        $INSTALL_CACHE_TOOL_DIR/upgrade-tool --device_index -1 --component uefi --path $INSTALL_CACHE_FILE_DIR/HI1910_FPGA_DDR.fd
        minirc_upgrade_progress "uefi flash" $? 100
        # show version after upgrade
        $INSTALL_CACHE_TOOL_DIR/upgrade-tool --device_index -1 --component -1 --all --version

        #mark firmware upgraded
        if [ $upgrade_result -eq 0 ]; then
            echo "[INFO]$(date +%Y-%m-%d,%H:%m:%s)  firmware upgraded success"
            echo "$(date +%Y-%m-%d,%H:%M:%S)  firmware upgraded" >> /var/mini_upgraded
            chmod -f 640 /var/mini_upgraded
            if [ -e $FW_UPGRADE_RESULT_FILE ]; then
                rm -f $FW_UPGRADE_RESULT_FILE
            fi
        else
            echo "[ERROR]$(date +%Y-%m-%d,%H:%m:%s)  firmware upgraded failed: $upgrade_result"
            echo "logfile: $INSTALL_CACHE_DIR/driver/firmware_upgrade_progress.log"
        fi
    fi
}

isRoot() {
    [ "$RUN_TYPE" == "pc" ] && return
    if [ $(id -u) -ne 0 ]; then
        echo "[ERROR]do not have root permission, operation failed, please use root permission!"
        exit 1
    fi
}

logOperation() {
    local operation=${LOG_OPERATION_UPGRADE}
    local start_time="$1"
    local runfilename=${PKG_FILE_NAME}
    local result="$2"
    local level=${LOG_LEVEL_MINOR}

    if [ ! -f "${OPERATION_LOGPATH}" ]; then
        touch ${OPERATION_LOGPATH}
        chmod 640 ${OPERATION_LOGPATH}
    fi
    echo "${operation} ${level} root ${start_time} 127.0.0.1 ${runfilename} ${result}" >> ${OPERATION_LOGPATH}
}

createTmpFile() {
    if [ -e $1/nve.tmp ]; then
        rm -rf $1/nve.tmp
    fi
    touch $1/nve.tmp
    echo RUN_TYPE=$2 >> $1/nve.tmp
    echo UPGRADE_NVE_LEVEL=$nve_level >> $1/nve.tmp
}

recursion_remove_file() {
    [ -d "$1" ] || return
    for tmp_file in `ls "$1"`
    do
        if [ -d "$1/${tmp_file}" ]; then
            recursion_remove_file "$1/${tmp_file}"
        elif grep "$1/${tmp_file}" ${old_filelist} >/dev/null 2>&1; then
            rm -f $1/${tmp_file} || log "[WARNING]rm $1/${tmp_file} failed, the file maybe in used"
        fi
    done
    [ -d "$1" ] && [ `ls "$1" | wc -l` -eq 0 ] && rm -rf $1
}

remove_drv_folder() {
    old_filelist="/var/davinci/scripts/filelist.csv"
    local tmp_dir="/usr/local/Ascend/driver"
    local tmp_libs_dir="/usr/local/Ascend/driver/lib64"
    local tmp_tools_dir="/usr/local/Ascend/driver/tools"
    if [ -e "${old_filelist}" ] && [[ -d "${tmp_libs_dir}" || -d "${tmp_tools_dir}" ]]; then
        recursion_remove_file ${tmp_libs_dir}
        recursion_remove_file ${tmp_tools_dir}
    fi
    [ -d "${tmp_dir}" ] && [ `ls "${tmp_dir}" | wc -l` -eq 0 ] && rm -rf ${tmp_dir}
    log "[INFO]remove old filelist finish"
}

INSTALL_CACHE_DIR="/var/mini/install_cache"
PKG_DIR="/opt/mini"
PKG_FILE=""
PKG_FILE_NAME=""
occupy_space="200"
start_time=$(date +"%Y-%m-%d %H:%M:%S")

if [[ "$1" =~ "mini_pkg_install" ]];then
    SDCARD_MOUNT_DIR=$1
    INSTALL_CACHE_DIR="$SDCARD_MOUNT_DIR/var/mini/install_cache"
    PKG_DIR="$SDCARD_MOUNT_DIR/opt/mini"
    RUN_TYPE="pc"
else
    SDCARD_MOUNT_DIR=""
    INSTALL_CACHE_DIR="/var/mini/install_cache"
    PKG_DIR="/opt/mini"
    RUN_TYPE="mini"
fi

isRoot
createLogFile
rotateLog
log "[INFO]mini first phase of upgrade start"
log "[INFO]PKG_DIR=$PKG_DIR"
log "[INFO]INSTALL_CACHE_DIR=$INSTALL_CACHE_DIR"
log "[INFO]RUN_TYPE=$RUN_TYPE"

UPGRADE_TARGET="DEFAULT"
FW_UPGRADE_RESULT_FILE="/var/davinci/firmware/upgrade_failed"
PKG_FILE_NUM_C7X=$(ls $PKG_DIR | grep ".tar.gz" | grep -c "A200dk-npu-driver")
if [ $RUN_TYPE == "mini" ]; then
    if [ $PKG_FILE_NUM_C7X -gt 1 ];then
        log "[ERROR]Detect more than one driver package in $PKG_DIR, please keep only one."
        exit 4
    fi
    FS_PARTITION=`cat /proc/cmdline | tr -s ' ' '\n' | grep "root=" | awk -F '=' '{print $2}'`
    if [[ "${FS_PARTITION}" =~ "p4" ]]; then
        log "[ERROR]Current boot partition is backup, do not support upgrade driver package!"
        exit 4
    fi
fi

if [ -e $PKG_DIR/A200dk-npu-driver*.tar.gz  ]; then
    UPGRADE_TARGET="C7X"
    PKG_FILE_NUM=$PKG_FILE_NUM_C7X
    PKG_FILE_NAME=$(ls $PKG_DIR | grep ".tar.gz" | grep "A200dk-npu-driver" | sort -rV | head -n 1)

    INSTALL_CACHE_FILE_DIR=$INSTALL_CACHE_DIR/driver
    mkdir -p $SDCARD_MOUNT_DIR/var/davinci/firmware
else
    log "can't not find $PKG_DIR/A200dk-npu-driver*.tar.gz, installation failed"
    exit 4
fi

if [ $RUN_TYPE == "mini" ]; then
    INSTALL_CACHE_TOOL_DIR=/var/davinci/driver
    if [ ! -e $INSTALL_CACHE_TOOL_DIR/upgrade-tool ]; then
        INSTALL_CACHE_TOOL_DIR=/var/davinci/firmware
        if [ ! -e $INSTALL_CACHE_TOOL_DIR/upgrade-tool ]; then
            echo "[ERROR]can not find upgrade-tool in firmware or driver!"
            exit 4
        fi
    fi

    if [[ $PKG_FILE_NAME =~ "18.04" ]];then
        PACKAGE_VERSION="18.04"
        # Compatible with the hisi driver package
        INSTALL_CACHE_TOOL_DIR=$INSTALL_CACHE_FILE_DIR
    elif [[ $PKG_FILE_NAME =~ "16.04" ]];then
        PACKAGE_VERSION="16.04"
    elif [[ $PKG_FILE_NAME =~ "2.9" ]];then
        PACKAGE_VERSION="2.0 (SP9)"
    else
        echo "[ERROR]unknown driver package version, not 18.04 or 16.04 !!!"
        exit 4
    fi
    SYSTEM_VERSION=`cat /etc/os-release | grep "VERSION=" | awk -F '"' '{print $2}'`
    if [[ "$SYSTEM_VERSION" =~ "$PACKAGE_VERSION" ]];then
        echo "[INFO]start install driver-$PACKAGE_VERSION"
    else
        echo "[ERROR]driver and ubuntu system do not match, driver version $PACKAGE_VERSION is incompatible with $SYSTEM_VERSION"
        exit 4
    fi
fi

trap "logOperation \"${start_time}\" \"${LOG_RESULT_FAILED}\";exit 1" INT QUIT KILL TERM

if [ $PKG_FILE_NUM -ge 1 ];then
    PKG_FILE=$PKG_DIR/$PKG_FILE_NAME

    check_space
    if [ $RUN_TYPE == "mini" ]; then
        # fixup c3x firmware upgrade flag file is not /var/mini_upgraded.
        echo "$(date +%Y-%m-%d,%H:%M:%S)  firmware start" > /var/mini_upgraded
        chmod -f 640 /var/mini_upgraded
        # create upgrade_failed first，it will be delete after firmware upgrade success.
        echo "$(date +%Y-%m-%d,%H:%m:%s) firmware upgrade failed" > $FW_UPGRADE_RESULT_FILE
        chmod -f 640 $FW_UPGRADE_RESULT_FILE
    fi
    extrack_package
    device_crl_check
    install_phase1
    if [ $RUN_TYPE == "mini" ] && [ "$UPGRADE_TARGET" != "C3X" ];then
        # add record firmware version before upgrade.
        $INSTALL_CACHE_TOOL_DIR/upgrade-tool --device_index -1 --component -1 --version >> $logFile
        cp $INSTALL_CACHE_FILE_DIR/scripts/minirc_boot.sh /var
        chmod 550 /var/minirc_boot.sh
        sync
    fi
    firmware_upgrade

    rm -f $PKG_FILE
fi

if [ $RUN_TYPE == "mini" ]; then
    # clean /usr/local/Ascend/driver/tools and lib64, the folder only using for minirc docker map
    remove_drv_folder

    chmod 755 /etc/rc.local
    if [ -e $FW_UPGRADE_RESULT_FILE ]; then
        log "[ERROR]driver upgrade error, please check the system environment or replace the driver package."
    else
        log "[INFO]the first phase of upgrade finish, please reboot"
    fi
    sed -i "/minirc_boot.sh/s/A200dk-npu-driver*.*/${PKG_FILE_NAME}/" /etc/rc.local
fi
createTmpFile $SDCARD_MOUNT_DIR/var $RUN_TYPE

sync
exit 0
