#!/bin/bash
ASCEND_SECLOG="/var/log/ascend_seclog"
logFile="${ASCEND_SECLOG}/ascend_run_servers.log"
usrCsvFile="/var/davinci/scripts/userfilelist.csv"
log() {
    cur_date=`date +"%Y-%m-%d %H:%M:%S"`
    echo "[driver] [$cur_date] "$1 >> $logFile
}

set_led_state()
{
    #stats  booting  upgrading finish
    state=$1
    if [ $state == "booting" ]; then
        echo "led3:1led4:0" > /var/davinci/driver/led
    elif [ $state == "upgrading" ]; then
        echo  "led3:2led4:2" > /var/davinci/driver/led
    elif [ $state == "finish" ]; then
        echo "led3:1led4:1" > /var/davinci/driver/led
    fi
}

###############################################
#
#            Pre Init.
#
###############################################
pre_init_services()
{
    set_led_state "booting"
    /var/minirc_led >& /dev/null &
    echo 1    4    1    7 > /proc/sys/kernel/printk
}


###############################################
#
#            Load Drivers.
#
###############################################
load_tee_daemon()
{
    log "[INFO]start TEE daemon"
    nice -n -10 su - ${username} -c "nohup /var/tee-supplicant >/dev/null 2>&1 &"
}

ko_check()
{
    ko=$1
    [ -e "${ko}" ] || { log "[ERROR]insmod $ko fail" && return; }

    insmod $ko
    if [ $? -ne 0 ]; then
        log "[ERROR]insmod $ko fail"
    else
        log "[INFO]insmod $ko succ"
    fi
}

kbox_ko_check()
{
    mkdir -p /var/log/kbox
    insmod /var/davinci/driver/kbox.ko product="minirc" config=0x5f
    if [ $? -ne 0 ]; then
        log "[ERROR]insmod kbox.ko fail"
    else
        log "[INFO]insmod kbox.ko succ"
    fi
}

load_device_modules()
{
    log "[INFO]load device modules"
    ko_check /var/davinci/driver/drv_typec_hw.ko
    ko_check /var/davinci/driver/drv_typec_fusb301.ko
    ko_check /var/davinci/driver/drv_typec_pericom30216.ko
    ko_check /var/davinci/driver/drv_typec_ptn5150h.ko
    ko_check /var/davinci/driver/drv_typec_tusb320.ko
    ko_check /var/davinci/driver/drv_usb_dwc3.ko
    ko_check /var/davinci/driver/mntn_bbox.ko
    ko_check /var/davinci/driver/tee_drv.ko
    load_tee_daemon
    ko_check /var/davinci/driver/drv_dfm.ko
    ko_check /var/davinci/driver/drv_nor_flash.ko
    ko_check /var/davinci/driver/drv_e2prom.ko
    ko_check /var/davinci/driver/drv_pca6416.ko
    echo /dev/mtd1 > /sys/module/hisi_nve/parameters/nve
    ko_check /var/davinci/driver/drv_pcie.ko
    ko_check /var/davinci/driver/ipc_drv.ko
    ko_check /var/davinci/driver/drv_davinci_intf.ko
    ko_check /var/davinci/driver/drv_devmng.ko
    ko_check /var/davinci/driver/drv_ascend_devmng.ko
    ko_check /var/davinci/driver/drv_devdrv.ko
    ko_check /var/davinci/driver/drv_platform.ko
    ko_check /var/davinci/driver/drv_dvpp.ko
    ko_check /var/davinci/driver/drv_log.ko
    ko_check /var/davinci/driver/drv_mdio.ko
    ko_check /var/davinci/driver/drv_lsw.ko
    ko_check /var/davinci/driver/drv_gmac.ko
    ko_check /var/davinci/driver/drv_prof.ko
    ko_check /var/davinci/driver/drv_fan.ko
    ko_check /var/davinci/driver/drv_pwm.ko
    ko_check /var/davinci/driver/drv_spi.ko
    ko_check /var/davinci/driver/drv_upgrade.ko
    ko_check /var/davinci/driver/drv_watchdog.ko
    ko_check /var/davinci/driver/ascend_event_sched.ko
    ko_check /var/davinci/driver/drv_ts.ko
    ko_check /var/davinci/driver/kbox_init.ko
    kbox_ko_check
    if [ -e ${usrCsvFile} ]; then
        log "[INFO]load usr device modules"
        file_list=`awk -F ',' '$2=="insmod" {print $4}' ${usrCsvFile}`
        n=1
        for ko in ${file_list[@]}
        do
            if [[ ${ko} =~ ".ko" ]];then
                ko_check ${ko}
            fi
        done
    fi
    log "[INFO]finish load device modules"
}

###############################################
#
#            Load Config
#
###############################################

load_slogd()
{
    slog_path="/usr/slog"
    mkdir -p $slog_path
    chmod -f 750 $slog_path
    grep "^$username" /etc/passwd >/dev/null 2>&1
    if [ $? -eq 0 ];then
        chown -RPf $username:$usergroup $slog_path
    fi

    log "[INFO]start slogd"
    sysctl -w kernel.dmesg_restrict=0 >/dev/null 2>&1
    sed -i 's/kernel.dmesg_restrict=1/kernel.dmesg_restrict=0/' /etc/sysctl.conf
    grep "^$username" /etc/passwd >/dev/null 2>&1
    if [ $? -eq 0 ];then
        su - ${username} -c "nohup /var/sklogd >/dev/null 2>&1 &"
        su - ${username} -c "nohup /var/slogd >/dev/null 2>&1 &"
        su - ${username} -c "nohup /var/log-daemon >/dev/null 2>&1 &"
    else
        nohup /var/sklogd >/dev/null 2>&1 &
        nohup /var/slogd >/dev/null 2>&1 &
        nohup /var/log-daemon >/dev/null 2>&1 &
    fi
}


###############################################
#
#            Load Services.
#
###############################################
set_hdc_config()
{
        sed -i "4d" /etc/hdcBasic.cfg
        sed -i "3a TRANS_TYPE              = 0" /etc/hdcBasic.cfg
        sleep 1

}
# hdcd is belong to HwHiAiUser, must Downgrade execution
load_hdcd()
{
    chmod -f 550 /sbin/hdcd
    if [ -d  /var/hdcd ];then
        rm -rf /var/hdcd
    fi
    su - ${username} -c "mkdir -p /home/${username}/hdcd >/dev/null 2>&1"
    su - ${username} -c "mkdir -p /home/${username}/hdcd/device0 >/dev/null 2>&1"
    su - ${username} -c "mkdir -p /home/${username}/hdcd/device0/upgrade >/dev/null 2>&1"
    su - ${username} -c "mkdir -p /home/${username}/hdcd/device0/etc >/dev/null 2>&1"
    chmod -R 750 /home/${username}/hdcd
    log "[INFO]start hdcd:"
    nice -n -20 su ${username} -c "/sbin/hdcd"
}

load_IDE_daemon()
{
    ##add gdb attach permission for HwHiAiUser
    if [ ! -f /etc/sysctl.d/10-ptrace.conf ];then
        if [ ! -d /etc/sysctl.d ];then
            mkdir -p /etc/sysctl.d
        fi
        echo "kernel.yama.ptrace_scope = 0" >> /etc/sysctl.d/10-ptrace.conf
    else
        result=`grep -rn "kernel.yama.ptrace_scope" /etc/sysctl.d/10-ptrace.conf`
        if [ "x$result" != x"" ];then
            sed -i '/kernel.yama.ptrace_scope/d' /etc/sysctl.d/10-ptrace.conf
            echo "kernel.yama.ptrace_scope = 0" >> /etc/sysctl.d/10-ptrace.conf
        else
            echo "kernel.yama.ptrace_scope = 0" >> /etc/sysctl.d/10-ptrace.conf
        fi
    fi
}
load_dmp_daemon()
{
    log "[INFO]start DMP daemon"
    mkdir -p /var/driver
    chmod -Rf 750 /var/driver
    chown -f ${username}:${usergroup} /var/driver
    chmod -f 750 /mnt
    chown ${username}:${usergroup} /mnt
    nice -n -20 su - ${username} -c "nohup /var/dmp_daemon -I -U 8087 >/dev/null 2>&1 &"
}

load_device_services()
{
    set_hdc_config
    load_slogd
    load_hdcd
    load_IDE_daemon
    load_dmp_daemon
    if [ -e /var/davinci/scripts/minirc_user_service.sh ]; then
        bash /var/davinci/scripts/minirc_user_service.sh
    fi
}

init_hugepages()
{
    log "[INFO]init the num of the hugepages"
    local TotalMem=`cat /proc/meminfo | grep MemTotal | awk '{ print $2 }'`
    local nr_hugepages=`cat /proc/cmdline | tr -s ' ' '\n' | grep nr_hugepages | awk -F '=' '{print $2}'`

    #nr_hugepages is the ratio of hugetlb memory to total available memory
    if [ "$nr_hugepages" -ge 0 -a "$nr_hugepages" -le 100 ] 2>/dev/null ;then
        if [ "$nr_hugepages" -ge 50 ] 2>/dev/null ;then
            log "[INFO]Customer' val is too big, reduce to 50%"
            nr_hugepages=50
        fi
        val=$((TotalMem * nr_hugepages / 100 / 2048))
        echo $val > /proc/sys/vm/nr_hugepages
        log "[INFO]init_hugepages set the nr_hugepages to $val"
        return
    fi
    # value not in [0-100], set to 25% default
    if [ "$nr_hugepages" -gt 100 ] 2>/dev/null ;then
        val=$((TotalMem * 25 / 100 / 2048))
        echo $val > /proc/sys/vm/nr_hugepages
        log "[INFO]Customer's value is invalidated, Default set the nr_hugepages to 25%($val)"
        return
    fi
}

disable_clear_hugepage()
{
    echo 0 > /proc/sys/vm/enable_clear_hugepage
}

init_mem_waterline()
{
    #disable panic on oom
    echo 0 > /proc/sys/vm/panic_on_oom
    echo 0 > /proc/sys/vm/enable_oom_killer

    #always enable mem overcommit
    echo 1 > /proc/sys/vm/overcommit_memory
}

init_file_pagecache()
{
    # mini-rc use SD to mount rootfs. When there's a lot of IO request, such as tar, logd, read/write
    # It may block the process. Should reduce the backgroud queue.
    # dirty_background_ratio(default 10) change to 1
    # dirty_ratio(default 20) change to 2

    echo 1 > /proc/sys/vm/dirty_background_ratio
    echo 2 > /proc/sys/vm/dirty_ratio
}

init_usermemory()
{
    echo "create usermemory cgroup in the memory virtual file system"

    #OOM create cgroup
    if [ ! -d "/sys/fs/cgroup/memory" ]; then
        mkdir /sys/fs/cgroup/memory
        mount -t cgroup -o memory memory /sys/fs/cgroup/memory
    fi

    mkdir /sys/fs/cgroup/memory/usermemory
    chmod 755 /sys/fs/cgroup/memory/usermemory
    echo 1 > /sys/fs/cgroup/memory/usermemory/memory.oom_control

    #aicpu cust op
    mkdir /sys/fs/cgroup/memory/usermemory/cust_aicpu
    chmod 755 /sys/fs/cgroup/memory/usermemory/cust_aicpu
    echo 1 > /sys/fs/cgroup/memory/usermemory/cust_aicpu/memory.oom_control
}

modify_usermem_limit()
{
    #get system free memory
    local free_mem=`cat /proc/meminfo  | grep MemFree | awk '{print $2}'`
    local cma_free=`cat /proc/meminfo | grep "CmaFree" | awk '{print $2}'`

    local mem_limit=$((free_mem * 1024 - 300 * 1024 * 1024 - cma_free * 1024))
    echo $((mem_limit)) > /sys/fs/cgroup/memory/usermemory/memory.limit_in_bytes
    echo $((mem_limit / 2)) > /sys/fs/cgroup/memory/usermemory/cust_aicpu/memory.limit_in_bytes
}

init_cpusets()
{
    log "[INFO]create CtrlCPU and AICPU cgroup in the cpuset virtual file system"
    local MaxCpuID=`cat /proc/cpuinfo | grep "processor" | tail -1 | tr -cd "[0-9]"`

    if [ ! -d "/sys/fs/cgroup" ]; then
        mount -t tmpfs cgroup_root /sys/fs/cgroup
    fi

    if [ ! -d "/sys/fs/cgroup/cpuset" ]; then
        mkdir /sys/fs/cgroup/cpuset
        mount -t cgroup -ocpuset cpuset /sys/fs/cgroup/cpuset
    fi

    mkdir /sys/fs/cgroup/cpuset/CtrlCPU
    mkdir /sys/fs/cgroup/cpuset/AICPU
    mkdir /sys/fs/cgroup/cpuset/DataCPU

    cat /sys/fs/cgroup/cpuset/cpuset.mems > /sys/fs/cgroup/cpuset/CtrlCPU/cpuset.mems
    cat /sys/fs/cgroup/cpuset/cpuset.mems > /sys/fs/cgroup/cpuset/AICPU/cpuset.mems

    cat /sys/bus/cpu/devices/cpu0/topology/core_siblings_list > /sys/fs/cgroup/cpuset/CtrlCPU/cpuset.cpus
    cat /sys/bus/cpu/devices/cpu$MaxCpuID/topology/core_siblings_list > /sys/fs/cgroup/cpuset/AICPU/cpuset.cpus
    # miniv1 has no DataCPU
    echo ' ' > /sys/fs/cgroup/cpuset/DataCPU/cpuset.cpus
}

attach_tasks()
{
    log "[INFO]make all non-AI tasks attach to CtrlCPU cgroup"
    while read line
    do
        echo $line >> /sys/fs/cgroup/cpuset/CtrlCPU/tasks 2> /dev/null
    done < /sys/fs/cgroup/cpuset/tasks
}

mount_disk() {
    log_path=/var/log/npu/slog/
    if [ ! -d $log_path ]; then
        rm $log_path >> /dev/null 2>&1
        mkdir -p $log_path
        chmod 750 /var/log/npu
        chmod 750 $log_path
        chown -h ${username}:${usergroup} /var/log/npu
        chown -RPf ${username}:${usergroup} $log_path
    fi
    grep -qs $log_path /proc/mounts >> /dev/null 2>&1
    ret=$?
    if [ -e "${devname}p2" ] && [ $ret -ne 0 ]; then
        mkdir /var/oldlog
        rsync -a --include '*/' --exclude '*'  $log_path /var/oldlog/
        mount "${devname}p2" $log_path
        rsync -a --include '*/' --exclude '*'  /var/oldlog/ $log_path
        rm -rf /var/oldlog
        chmod -f 750 $log_path && chown -RPf ${username}:${usergroup} $log_path
    fi

    grep -qs '/home' /proc/mounts >> /dev/null 2>&1
    ret=$?
    if [ -e "${devname}p3" ] && [ $ret -ne 0 ]; then
        mount "${devname}p3" /home
    fi
}
change_dev_mode()
{
    cd /dev/
    chmod -f 600 hisi_bbox*
    chmod -f 660 davinci0 davinci_manager dbg_drv devmm_svm hi_dvpp log_drv nve prof_drv svm0 random urandom upgrade PCA6416 pwm eeprom_m24256* hi_sfc i2c0_slave i2c-0 event_sched ts_aisle ascend_manager
    chmod -f 640 tee0 teepriv0
    chown -f ${username}:${usergroup} davinci0 davinci_manager dbg_drv devmm_svm hi_dvpp hisi_bbox* log_drv nve prof_drv svm0 tee0 teepriv0 random urandom upgrade PCA6416 pwm eeprom_m24256* hi_sfc i2c0_slave i2c-0 event_sched ts_aisle ascend_manager
    cd /sys/kernel/
    chmod -f 0755 debug
    cd debug
    chmod -f 0755 fan
    chmod -f 0444 /sys/kernel/debug/fan/fan_speed
    chown -f ${username}:${usergroup} /sys/kernel/debug/fan/fan_speed
    cd /dev/
}
change_proc_mode() {
    if [ -d /proc/flash ]; then
        chmod -f 550 /proc/flash
        chown -f ${username}:${usergroup} /proc/flash
        chmod -f 440 /proc/flash/*
        chown -RPf ${username}:${usergroup} /proc/flash/*
    fi
    if [ -f /proc/mtd ]; then
        chmod -f 440 /proc/mtd
        chown -f ${username}:${usergroup} /proc/mtd
    fi
    return 0
}

config_time_file()
{
    touch /etc/localtime
    chown -R ${username}:${usergroup} /etc/localtime
    chmod -R 640 /etc/localtime
}

load_soft_link()
{
    if [ ! -d /usr/lib64/aicpu_kernels ];then
        mkdir /usr/lib64/aicpu_kernels
    fi
    chown HwHiAiUser:HwHiAiUser /usr/lib64/aicpu_kernels
    ln -sf /usr/lib64/aicpu_kernels/0/aicpu_kernels_device/libtensorflow.so  /usr/lib64/libtensorflow.so
    ln -sf /usr/lib64/aicpu_kernels/libtorch_cpu.so /usr/lib64/libtorch_cpu.so
    ln -sf /usr/lib64/aicpu_kernels/libc10.so /usr/lib64/libc10.so
    ln -sf /usr/lib64/aicpu_kernels/0/libcpu_kernels_context.so /usr/lib64/libcpu_kernels_context.so
}

hdc_ppc_create()
{
    if [ ! -d /var/hdc_ppc ];then
        mkdir -p /var/hdc_ppc
    fi
    chown -R ${username}:${usergroup} /var/hdc_ppc
    chmod -R 750 /var/hdc_ppc
}

get_boot_partition() {
    boot_cmd_path=/proc/cmdline
    if [ -e ${boot_cmd_path} ]; then
        boot_area=`cat /proc/cmdline | awk '{for(i=1;i<=NF;i++){split($i,cmd_str,"="); if(cmd_str[1]=="slaveboot"){print cmd_str[2]}}}'`
        fs_partition=`cat /proc/cmdline | tr -s ' ' '\n' | grep "root=" | awk -F '=' '{print $2}'`
        if [ $boot_area -eq 1 ]; then
            partition_flag="backup"
        fi
    fi
}

system_boot_check() {
    if [ -e $FW_UPGRADE_RESULT_FILE ] && [ "$partition_flag" = "main" ]; then
        log "[WARN]last firmware upgrade failed, wait boot from backup partition."
        exit 0
    fi
}

update_aicpu_config()
{
    DEVDRV_AICPU_CONFIG="/sys/class/devdrv_manager/davinci_manager/devdrv_manager/devdrv_aicpu_config"
    if [ -e "${DEVDRV_AICPU_CONFIG}" ];then
        local AICPU_CONFIG=`cat ${DEVDRV_AICPU_CONFIG}`
        if [ "$AICPU_CONFIG"x != "aicpu_config-invalid"x ]; then
            cat ${DEVDRV_AICPU_CONFIG} > /sys/fs/cgroup/cpuset/AICPU/cpuset.cpus
        fi
        chmod 440 ${DEVDRV_AICPU_CONFIG}
    fi

    DEVDRV_CTRLCPU_CONFIG="/sys/class/devdrv_manager/davinci_manager/devdrv_manager/devdrv_ctrlcpu_config"
    if [ -e "${DEVDRV_CTRLCPU_CONFIG}" ];then
        local CTRLCPU_CONFIG=`cat ${DEVDRV_CTRLCPU_CONFIG}`
        if [ "$CTRLCPU_CONFIG"x != "ctrlcpu_config-invalid"x ]; then
            cat ${DEVDRV_CTRLCPU_CONFIG} > /sys/fs/cgroup/cpuset/CtrlCPU/cpuset.cpus
            # miniv1 has no DataCPU
            echo ' ' > /sys/fs/cgroup/cpuset/DataCPU/cpuset.cpus
        fi
        chmod 440 ${DEVDRV_CTRLCPU_CONFIG}
    fi
}

get_devname() {
    local tmp_name=`cat /proc/cmdline | tr -s ' ' '\n' | grep "root=" | awk -F '=' '{print $2}' | awk -F 'p' '{print $1}'`
    if [ -e "${tmp_name}" ]; then
        devname=${tmp_name}
    fi
}

fs_process() {
    # reserver for inotifywait
    if [ ! -e "${sync_scripts}" ];then
        log "[ERROR]Sync scripts(${sync_scripts}) does not exist."
	return 1
    fi

    ${sync_scripts} /var/davinci/scripts/backup_file_list.conf &
    return 0
}

fs_file_sync() {
    # filelist
    ${backup_scripts}  || return 1

    # clean backup install_cache
    if [[ "${fs_partition}" == "${devname}p1" && -d "${backup_install_cache}" ]]; then
        rm -rf ${backup_install_cache}
        # reserve for rm other flag
    fi

    return 0
}

fs_sync() {
    # check if support fs backup
    [[ `/var/davinci/driver/upgrade-tool --fs_backup_status | grep enable` ]] || return 0

    local fs_part=("${devname}p1" "${devname}p4")

    # mount the other partition
    [ -d "${fs_partition_name}" ] || mkdir -p ${fs_partition_name}
    if [ "${fs_partition}" == "${devname}p1" ]; then
        mount "${devname}p4" ${fs_partition_name}
    else
        mount "${devname}p1" ${fs_partition_name}
    fi

    FEATURE_FS_FILE_SYNC=y

    [ -e "${feature_file}" ] && source ${feature_file}
    if [ "${FEATURE_FS_FILE_SYNC}" != "y" ]; then
        log "[INFO]FEATURE_FS_FILE_SYNC=${FEATURE_FS_FILE_SYNC}, main and backup partition do not sync"
        return 0
    fi

    # check main/backup os-release
    which diff >/dev/null 2>&1
    if [ $? -eq 0 ]; then
        diff /etc/os-release "${fs_partition_name}/etc/os-release" >/dev/null 2>&1 || \
            { log "[WARNING]Main and backup partition os-release is different, do not sync." && return 0; }
    else
        cur_osver_md5=`md5sum /etc/os-release | awk -F ' ' '{print $1}'`
        backup_osver_md5=`md5sum "${fs_partition_name}/etc/os-release" | awk -F ' ' '{print $1}'`
        [ "${cur_osver_md5}" == "${backup_osver_md5}" ] || \
            { log "[WARNING]Main and backup partition os-release is different, do not sync." && return 0; }
    fi

    log "[INFO]Filesystem sync start."
    # copy driver file, detail in filelist and copylist
    fs_file_sync || return 1

    # load process, inotifywait
    fs_process || return 1
    log "[INFO]Filesystem sync finished."
}

fs_boot_warn()
{
    if [ "${fs_partition}" == "${devname}p4" ]; then
        grep -q "current boot filesystem is backup partition" /etc/profile || \
            echo "echo \"warning: current boot filesystem is backup partition.\"" >> /etc/profile
    fi
}

#set environment variable
set_environment_variable()
{
    file_conf=/etc/ld.so.conf.d/mind_so.conf
    if [ -f $file_conf ]; then
        rm $file_conf
        ldconfig
    fi
    echo /usr/lib64 >> $file_conf
    ldconfig
}

###############################################
#
#            main process.
#
###############################################
username=HwHiAiUser
usergroup=HwHiAiUser
devname="/dev/mmcblk1"
fs_partition_name="/data/fs_backup"
backup_install_cache="${fs_partition_name}/var/mini/install_cache"
backup_scripts="/var/davinci/scripts/minirc_driver_backup.sh"
sync_scripts="/var/davinci/scripts/minirc_file_sync.sh"
feature_file="/var/davinci/scripts/feature.conf"
rm -rf /home/HwHiAiUser/tmp

if [ -f /var/davinci/driver/version.info ]; then
    . /var/davinci/driver/version.info
    log "current version: $Version"
fi

FW_UPGRADE_RESULT_FILE="/var/davinci/firmware/upgrade_failed"
partition_flag="main"
fs_partition="${devname}p1"
get_boot_partition
system_boot_check

log "pre_init_services"
get_devname
mount_disk
pre_init_services
log "load device modules"
init_hugepages
disable_clear_hugepage
init_mem_waterline
init_file_pagecache
load_soft_link
set_environment_variable
init_cpusets
init_usermemory
attach_tasks
load_device_modules
if [ -e /sbin/ifdown ]; then
    ifdown eth0
else
    ifconfig eth0 down
fi
update_aicpu_config
#services leads on driver chown
change_dev_mode
hdc_ppc_create

load_device_services
attach_tasks
sh /var/usb_ether.sh
config_time_file

#boot success, sync sd/emmc component
/var/davinci/driver/upgrade-tool --sync
log "sync sd/emmc: $?"

fs_boot_warn

# filesystem sync and start related process when fs backup is supported
fs_sync

rm -f $FW_UPGRADE_RESULT_FILE
if [ -e /var/mini_upgraded ]; then
    # delete old driver dir
    if [ -d /var/davinci/firmware ]; then
        rm -rf /var/davinci/firmware
    fi
fi

# del another partition upgrade_failed
[ -e "${fs_partition_name}/$FW_UPGRADE_RESULT_FILE" ] && \
    rm -f "${fs_partition_name}/$FW_UPGRADE_RESULT_FILE"

chmod -f 666 /sys/fs/cgroup/cpuset/AICPU/tasks
chmod -f 640 /var/davinci/driver/led
chmod -f a+w /var/
chmod -f 666 /dev/mcc_userdev
chmod -f 660 /dev/pcie_dma
chown -f ${username}:${usergroup} /dev/pcie_dma
chmod -f 640 $logFile
change_dev_mode
change_proc_mode
modify_usermem_limit

exit 0
